import datetime
from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="SharedLinkEditDto")


@_attrs_define
class SharedLinkEditDto:
    """
    Attributes:
        allow_download (Union[Unset, bool]):
        allow_upload (Union[Unset, bool]):
        change_expiry_time (Union[Unset, bool]): Few clients cannot send null to set the expiryTime to never.
            Setting this flag and not sending expiryAt is considered as null instead.
            Clients that can send null values can ignore this.
        description (Union[None, Unset, str]):
        expires_at (Union[None, Unset, datetime.datetime]):
        password (Union[None, Unset, str]):
        show_metadata (Union[Unset, bool]):
        slug (Union[None, Unset, str]):
    """

    allow_download: Union[Unset, bool] = UNSET
    allow_upload: Union[Unset, bool] = UNSET
    change_expiry_time: Union[Unset, bool] = UNSET
    description: Union[None, Unset, str] = UNSET
    expires_at: Union[None, Unset, datetime.datetime] = UNSET
    password: Union[None, Unset, str] = UNSET
    show_metadata: Union[Unset, bool] = UNSET
    slug: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        allow_download = self.allow_download

        allow_upload = self.allow_upload

        change_expiry_time = self.change_expiry_time

        description: Union[None, Unset, str]
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        expires_at: Union[None, Unset, str]
        if isinstance(self.expires_at, Unset):
            expires_at = UNSET
        elif isinstance(self.expires_at, datetime.datetime):
            expires_at = self.expires_at.isoformat()
        else:
            expires_at = self.expires_at

        password: Union[None, Unset, str]
        if isinstance(self.password, Unset):
            password = UNSET
        else:
            password = self.password

        show_metadata = self.show_metadata

        slug: Union[None, Unset, str]
        if isinstance(self.slug, Unset):
            slug = UNSET
        else:
            slug = self.slug

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if allow_download is not UNSET:
            field_dict["allowDownload"] = allow_download
        if allow_upload is not UNSET:
            field_dict["allowUpload"] = allow_upload
        if change_expiry_time is not UNSET:
            field_dict["changeExpiryTime"] = change_expiry_time
        if description is not UNSET:
            field_dict["description"] = description
        if expires_at is not UNSET:
            field_dict["expiresAt"] = expires_at
        if password is not UNSET:
            field_dict["password"] = password
        if show_metadata is not UNSET:
            field_dict["showMetadata"] = show_metadata
        if slug is not UNSET:
            field_dict["slug"] = slug

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        allow_download = d.pop("allowDownload", UNSET)

        allow_upload = d.pop("allowUpload", UNSET)

        change_expiry_time = d.pop("changeExpiryTime", UNSET)

        def _parse_description(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        description = _parse_description(d.pop("description", UNSET))

        def _parse_expires_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                expires_at_type_0 = isoparse(data)

                return expires_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        expires_at = _parse_expires_at(d.pop("expiresAt", UNSET))

        def _parse_password(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        password = _parse_password(d.pop("password", UNSET))

        show_metadata = d.pop("showMetadata", UNSET)

        def _parse_slug(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        slug = _parse_slug(d.pop("slug", UNSET))

        shared_link_edit_dto = cls(
            allow_download=allow_download,
            allow_upload=allow_upload,
            change_expiry_time=change_expiry_time,
            description=description,
            expires_at=expires_at,
            password=password,
            show_metadata=show_metadata,
            slug=slug,
        )

        shared_link_edit_dto.additional_properties = d
        return shared_link_edit_dto

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="PersonCreateDto")


@_attrs_define
class PersonCreateDto:
    """
    Attributes:
        birth_date (Union[None, Unset, datetime.date]): Person date of birth.
            Note: the mobile app cannot currently set the birth date to null.
        color (Union[None, Unset, str]):
        is_favorite (Union[Unset, bool]):
        is_hidden (Union[Unset, bool]): Person visibility
        name (Union[Unset, str]): Person name.
    """

    birth_date: Union[None, Unset, datetime.date] = UNSET
    color: Union[None, Unset, str] = UNSET
    is_favorite: Union[Unset, bool] = UNSET
    is_hidden: Union[Unset, bool] = UNSET
    name: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        birth_date: Union[None, Unset, str]
        if isinstance(self.birth_date, Unset):
            birth_date = UNSET
        elif isinstance(self.birth_date, datetime.date):
            birth_date = self.birth_date.isoformat()
        else:
            birth_date = self.birth_date

        color: Union[None, Unset, str]
        if isinstance(self.color, Unset):
            color = UNSET
        else:
            color = self.color

        is_favorite = self.is_favorite

        is_hidden = self.is_hidden

        name = self.name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if birth_date is not UNSET:
            field_dict["birthDate"] = birth_date
        if color is not UNSET:
            field_dict["color"] = color
        if is_favorite is not UNSET:
            field_dict["isFavorite"] = is_favorite
        if is_hidden is not UNSET:
            field_dict["isHidden"] = is_hidden
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)

        def _parse_birth_date(data: object) -> Union[None, Unset, datetime.date]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                birth_date_type_0 = isoparse(data).date()

                return birth_date_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.date], data)

        birth_date = _parse_birth_date(d.pop("birthDate", UNSET))

        def _parse_color(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        color = _parse_color(d.pop("color", UNSET))

        is_favorite = d.pop("isFavorite", UNSET)

        is_hidden = d.pop("isHidden", UNSET)

        name = d.pop("name", UNSET)

        person_create_dto = cls(
            birth_date=birth_date,
            color=color,
            is_favorite=is_favorite,
            is_hidden=is_hidden,
            name=name,
        )

        person_create_dto.additional_properties = d
        return person_create_dto

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="SessionUnlockDto")


@_attrs_define
class SessionUnlockDto:
    """
    Attributes:
        password (Union[Unset, str]):
        pin_code (Union[Unset, str]):  Example: 123456.
    """

    password: Union[Unset, str] = UNSET
    pin_code: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        password = self.password

        pin_code = self.pin_code

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if password is not UNSET:
            field_dict["password"] = password
        if pin_code is not UNSET:
            field_dict["pinCode"] = pin_code

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        password = d.pop("password", UNSET)

        pin_code = d.pop("pinCode", UNSET)

        session_unlock_dto = cls(
            password=password,
            pin_code=pin_code,
        )

        session_unlock_dto.additional_properties = d
        return session_unlock_dto

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.source_type import SourceType
from ..types import UNSET, Unset

T = TypeVar("T", bound="AssetFaceWithoutPersonResponseDto")


@_attrs_define
class AssetFaceWithoutPersonResponseDto:
    """
    Attributes:
        bounding_box_x1 (int):
        bounding_box_x2 (int):
        bounding_box_y1 (int):
        bounding_box_y2 (int):
        id (UUID):
        image_height (int):
        image_width (int):
        source_type (Union[Unset, SourceType]):
    """

    bounding_box_x1: int
    bounding_box_x2: int
    bounding_box_y1: int
    bounding_box_y2: int
    id: UUID
    image_height: int
    image_width: int
    source_type: Union[Unset, SourceType] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        bounding_box_x1 = self.bounding_box_x1

        bounding_box_x2 = self.bounding_box_x2

        bounding_box_y1 = self.bounding_box_y1

        bounding_box_y2 = self.bounding_box_y2

        id = str(self.id)

        image_height = self.image_height

        image_width = self.image_width

        source_type: Union[Unset, str] = UNSET
        if not isinstance(self.source_type, Unset):
            source_type = self.source_type.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "boundingBoxX1": bounding_box_x1,
                "boundingBoxX2": bounding_box_x2,
                "boundingBoxY1": bounding_box_y1,
                "boundingBoxY2": bounding_box_y2,
                "id": id,
                "imageHeight": image_height,
                "imageWidth": image_width,
            }
        )
        if source_type is not UNSET:
            field_dict["sourceType"] = source_type

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        bounding_box_x1 = d.pop("boundingBoxX1")

        bounding_box_x2 = d.pop("boundingBoxX2")

        bounding_box_y1 = d.pop("boundingBoxY1")

        bounding_box_y2 = d.pop("boundingBoxY2")

        id = UUID(d.pop("id"))

        image_height = d.pop("imageHeight")

        image_width = d.pop("imageWidth")

        _source_type = d.pop("sourceType", UNSET)
        source_type: Union[Unset, SourceType]
        if isinstance(_source_type, Unset):
            source_type = UNSET
        else:
            source_type = SourceType(_source_type)

        asset_face_without_person_response_dto = cls(
            bounding_box_x1=bounding_box_x1,
            bounding_box_x2=bounding_box_x2,
            bounding_box_y1=bounding_box_y1,
            bounding_box_y2=bounding_box_y2,
            id=id,
            image_height=image_height,
            image_width=image_width,
            source_type=source_type,
        )

        asset_face_without_person_response_dto.additional_properties = d
        return asset_face_without_person_response_dto

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Main CLI entry point for immichporter package."""

import click
import sys

# Import subcommands
from immichporter.gphotos.commands import cli_gphotos
from immichporter.db.commands import cli_db
from immichporter.immich.commands import cli_immich

from loguru import logger
from rich.console import Console
from immichporter.gphotos.utils import traceback

console = Console()


@click.group(context_settings={"help_option_names": ["-h", "--help"]})
@click.version_option()
def cli():
    """Immichporter retrieves metadata not available in google takeout,
    including shared albums, assets, and shared users.
    You can use this data to update assets in Immich,
    re-add users to shared albums, and even move assets to their correct owners.

    The command line interface uses subcommands for the various tasks (see below).

    This are the steps needed to extract the information from google photos and update immich:

    \b
        immich-go ... # see https://github.com/simulot/immich-go
        immichporter gphotos login
        immichporter gphotos albums
        immichporter gphotos photos
        immichporter db edit-users
        export IMMICH_ENDPOINT=http://localhost:2283
        export IMMICH_API_KEY=your_api_key
        export IMMICH_INSECURE=1
        immichporter immich update-users
        immichporter immich update-albums
        immichporter immich sync-albums
        immichporter immich adjust-owners

    """
    pass


cli.add_command(cli_gphotos)
cli.add_command(cli_db)
cli.add_command(cli_immich)


def handle_keyboard_interrupt(exc_type, exc_value, exc_traceback):
    if exc_type is KeyboardInterrupt:
        console.print("[red]Abort[/red]")
        logger.debug(traceback(exc_value))
        sys.exit(1)
    else:
        logger.debug(traceback(exc_value))
        console.print("[red]Exception [dim]" + str(exc_value) + "[/]")
        console.print(
            "[dim]Use [i][yellow]-l debug[/yellow][/i] to see traceback[/dim]"
        )
        sys.exit(1)
        # sys.__excepthook__(exc_type, exc_value, exc_traceback)


sys.excepthook = handle_keyboard_interrupt


def main():
    """Main entry point."""
    cli()


if __name__ == "__main__":
    main()



import datetime
from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
    cast,
)
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.shared_link_type import SharedLinkType
from ..types import UNSET, Unset

T = TypeVar("T", bound="SharedLinkCreateDto")


@_attrs_define
class SharedLinkCreateDto:
    """
    Attributes:
        type_ (SharedLinkType):
        album_id (Union[Unset, UUID]):
        allow_download (Union[Unset, bool]):  Default: True.
        allow_upload (Union[Unset, bool]):
        asset_ids (Union[Unset, list[UUID]]):
        description (Union[None, Unset, str]):
        expires_at (Union[None, Unset, datetime.datetime]):
        password (Union[None, Unset, str]):
        show_metadata (Union[Unset, bool]):  Default: True.
        slug (Union[None, Unset, str]):
    """

    type_: SharedLinkType
    album_id: Union[Unset, UUID] = UNSET
    allow_download: Union[Unset, bool] = True
    allow_upload: Union[Unset, bool] = UNSET
    asset_ids: Union[Unset, list[UUID]] = UNSET
    description: Union[None, Unset, str] = UNSET
    expires_at: Union[None, Unset, datetime.datetime] = UNSET
    password: Union[None, Unset, str] = UNSET
    show_metadata: Union[Unset, bool] = True
    slug: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_.value

        album_id: Union[Unset, str] = UNSET
        if not isinstance(self.album_id, Unset):
            album_id = str(self.album_id)

        allow_download = self.allow_download

        allow_upload = self.allow_upload

        asset_ids: Union[Unset, list[str]] = UNSET
        if not isinstance(self.asset_ids, Unset):
            asset_ids = []
            for asset_ids_item_data in self.asset_ids:
                asset_ids_item = str(asset_ids_item_data)
                asset_ids.append(asset_ids_item)

        description: Union[None, Unset, str]
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        expires_at: Union[None, Unset, str]
        if isinstance(self.expires_at, Unset):
            expires_at = UNSET
        elif isinstance(self.expires_at, datetime.datetime):
            expires_at = self.expires_at.isoformat()
        else:
            expires_at = self.expires_at

        password: Union[None, Unset, str]
        if isinstance(self.password, Unset):
            password = UNSET
        else:
            password = self.password

        show_metadata = self.show_metadata

        slug: Union[None, Unset, str]
        if isinstance(self.slug, Unset):
            slug = UNSET
        else:
            slug = self.slug

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type_,
            }
        )
        if album_id is not UNSET:
            field_dict["albumId"] = album_id
        if allow_download is not UNSET:
            field_dict["allowDownload"] = allow_download
        if allow_upload is not UNSET:
            field_dict["allowUpload"] = allow_upload
        if asset_ids is not UNSET:
            field_dict["assetIds"] = asset_ids
        if description is not UNSET:
            field_dict["description"] = description
        if expires_at is not UNSET:
            field_dict["expiresAt"] = expires_at
        if password is not UNSET:
            field_dict["password"] = password
        if show_metadata is not UNSET:
            field_dict["showMetadata"] = show_metadata
        if slug is not UNSET:
            field_dict["slug"] = slug

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        type_ = SharedLinkType(d.pop("type"))

        _album_id = d.pop("albumId", UNSET)
        album_id: Union[Unset, UUID]
        if isinstance(_album_id, Unset):
            album_id = UNSET
        else:
            album_id = UUID(_album_id)

        allow_download = d.pop("allowDownload", UNSET)

        allow_upload = d.pop("allowUpload", UNSET)

        asset_ids = []
        _asset_ids = d.pop("assetIds", UNSET)
        for asset_ids_item_data in _asset_ids or []:
            asset_ids_item = UUID(asset_ids_item_data)

            asset_ids.append(asset_ids_item)

        def _parse_description(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        description = _parse_description(d.pop("description", UNSET))

        def _parse_expires_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                expires_at_type_0 = isoparse(data)

                return expires_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        expires_at = _parse_expires_at(d.pop("expiresAt", UNSET))

        def _parse_password(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        password = _parse_password(d.pop("password", UNSET))

        show_metadata = d.pop("showMetadata", UNSET)

        def _parse_slug(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        slug = _parse_slug(d.pop("slug", UNSET))

        shared_link_create_dto = cls(
            type_=type_,
            album_id=album_id,
            allow_download=allow_download,
            allow_upload=allow_upload,
            asset_ids=asset_ids,
            description=description,
            expires_at=expires_at,
            password=password,
            show_metadata=show_metadata,
            slug=slug,
        )

        shared_link_create_dto.additional_properties = d
        return shared_link_create_dto

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Google Photos CLI commands."""

import asyncio
import click
from rich.console import Console

from loguru import logger

from immichporter.database import get_db_session, get_database_stats, init_database
from immichporter.gphotos.scraper import GooglePhotosScraper
from immichporter.gphotos.settings import playwright_session_dir
from immichporter.commands import logging_options, database_options

# Create a Click command group
cli_gphotos = click.Group(
    "gphotos", help="Login to Google Photos and export albums and photos metadata"
)

console = Console()


# Common options


def album_options(func):
    """Album options. Use variables `max_albums`, `start_album`, `start_album_fresh`, and `album_ids` in your function."""
    func = click.option(
        "-m",
        "--max-albums",
        default=0,
        help="Maximum number of albums to processm, default: all albums",
    )(func)
    func = click.option(
        "-s",
        "--start-album",
        default=1,
        help="Start processing from this album position (1-based)",
    )(func)
    func = click.option(
        "-f",
        "--start-album-fresh",
        is_flag=True,
        help="Start processing from the beginning, ignoring existing albums",
    )(func)
    return func


def playwright_options(func):
    """Playwright options. Use variables `profile_dir` and `clear_storage` in your function."""
    func = click.option(
        "-p",
        "--profile-dir",
        envvar="PROFILE_DIR",
        show_envvar=True,
        default=str(playwright_session_dir),
        help="Path to store browser profile data",
    )(func)
    func = click.option(
        "-x",
        "--clear-storage",
        is_flag=True,
        help="Clear browser storage before starting",
    )(func)
    return func


def playwright_headless_options(func):
    """Playwright options. Use variable `show_browser` in your function."""
    func = click.option(
        "-b",
        "--show-browser",
        is_flag=True,
        help="Show browser",
    )(func)
    return func


# Common scraper setup
async def setup_scraper(
    db_path="immichporter.db",
    reset_db=False,
    log_level="warning",
    profile_dir=playwright_session_dir,
    max_albums=0,
    start_album=1,
    album_fresh=False,
    clear_storage=False,
    headless=True,
):
    logger.info(f"Database path: {db_path}")

    # Create scraper instance
    scraper = GooglePhotosScraper(
        max_albums=max_albums,
        start_album=start_album,
        album_fresh=album_fresh,
        clear_storage=clear_storage,
        user_data_dir=profile_dir,
        headless=headless,
    )

    init_database(reset_db=reset_db)

    try:
        await scraper.setup_browser()
        return scraper
    except Exception as e:
        logger.error(f"Error setting up browser: {e}")
        await scraper.close()
        raise


@cli_gphotos.command()
@logging_options
@playwright_options
def login(log_level, clear_storage, profile_dir):
    """Login to Google Photos and save the session."""

    async def run_scraper_login():
        scraper = await setup_scraper(
            log_level=log_level,
            profile_dir=profile_dir,
            clear_storage=clear_storage,
            headless=False,
        )

        try:
            await scraper.login()
        except Exception as e:
            console.print(f"[red]Error: {e}[/red]")
        finally:
            await scraper.close()

    asyncio.run(run_scraper_login())


@cli_gphotos.command()
@album_options
@database_options
@logging_options
@playwright_options
@playwright_headless_options
def albums(
    max_albums,
    start_album,
    start_album_fresh,
    db_path,
    reset_db,
    log_level,
    clear_storage,
    profile_dir,
    show_browser,
):
    """List and export albums from Google Photos."""
    max_albums = max_albums if max_albums > 0 else 100000

    if start_album < 1:
        raise click.UsageError("Start album must be 1 or higher")

    async def run_scraper():
        scraper = await setup_scraper(
            db_path=db_path,
            reset_db=reset_db,
            log_level=log_level,
            profile_dir=profile_dir,
            max_albums=max_albums,
            start_album=start_album,
            album_fresh=start_album_fresh,
            clear_storage=clear_storage,
            headless=not show_browser,
        )

        try:
            logger.info("Collecting albums...")
            albums = await scraper.collect_albums(
                max_albums=max_albums,
                start_album=start_album if not start_album_fresh else 1,
            )
            console.print(f"[green]Collected {len(albums)} albums[/green]")

            # Show database stats
            with get_db_session() as session:
                stats = get_database_stats(session)
                console.print("\n[bold green]=== Database Statistics ===[/bold green]")
                console.print(f"[green]Total albums: {stats['total_albums']}[/green]")
                console.print(f"[green]Total photos: {stats['total_photos']}[/green]")
                console.print(f"[green]Total users: {stats['total_users']}[/green]")
                console.print(f"[green]Total errors: {stats['total_errors']}[/green]")
        except Exception as e:
            console.print(f"[red]Error: {e}[/red]")
        finally:
            await scraper.close()

    asyncio.run(run_scraper())


@cli_gphotos.command()
@album_options
@click.option(
    "-a",
    "--album-id",
    type=int,
    multiple=True,
    help="Specific album ID to process (overrides max_albums and start_album)",
)
@click.option(
    "-n",
    "--not-finished",
    is_flag=True,
    help="Process only albums that have not been finished yet",
)
@database_options
@logging_options
@playwright_options
@playwright_headless_options
def photos(
    max_albums,
    start_album,
    start_album_fresh,
    album_id,
    not_finished,
    db_path,
    reset_db,
    log_level,
    clear_storage,
    profile_dir,
    show_browser,
):
    """Export photos from Google Photos albums.

    By default, processes all albums. Use --album-id to process specific albums.
    """
    max_albums = max_albums if max_albums > 0 else None
    album_ids = album_id if album_id else None

    if not album_ids:  # Only validate start_album if not using album_ids
        if start_album < 1:
            raise click.UsageError("Start album must be 1 or higher")
    elif start_album_fresh or start_album > 1:
        console.print(
            "[yellow]Note: --start-album and --start-album-fresh are ignored when using --album-id[/yellow]"
        )

    async def run_scraper():
        scraper = await setup_scraper(
            db_path=db_path,
            reset_db=reset_db,
            log_level=log_level,
            profile_dir=profile_dir,
            max_albums=max_albums,
            start_album=start_album,
            album_fresh=start_album_fresh,
            clear_storage=clear_storage,
            headless=not show_browser,
        )

        try:
            logger.info("Starting photo export for all albums...")
            await scraper.scrape_albums_from_db(
                max_albums=max_albums,
                start_album=start_album if not start_album_fresh else 1,
                album_ids=album_ids,
                not_finished=not_finished,
                skip_existing=not start_album_fresh,
            )

            # Show database stats
            with get_db_session() as session:
                stats = get_database_stats(session)
                console.print("\n[bold green]=== Export Complete ===[/bold green]")
                console.print(f"[green]Total albums: {stats['total_albums']}[/green]")
                console.print(f"[green]Total photos: {stats['total_photos']}[/green]")
                console.print(f"[green]Total errors: {stats['total_errors']}[/green]")

        except Exception as e:
            console.print(f"[red]Error during export: {e}[/red]")
            raise
        finally:
            await scraper.close()

    asyncio.run(run_scraper())



import datetime
from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.asset_order import AssetOrder

T = TypeVar("T", bound="SyncAlbumV1")


@_attrs_define
class SyncAlbumV1:
    """
    Attributes:
        created_at (datetime.datetime):
        description (str):
        id (str):
        is_activity_enabled (bool):
        name (str):
        order (AssetOrder):
        owner_id (str):
        thumbnail_asset_id (Union[None, str]):
        updated_at (datetime.datetime):
    """

    created_at: datetime.datetime
    description: str
    id: str
    is_activity_enabled: bool
    name: str
    order: AssetOrder
    owner_id: str
    thumbnail_asset_id: Union[None, str]
    updated_at: datetime.datetime
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        created_at = self.created_at.isoformat()

        description = self.description

        id = self.id

        is_activity_enabled = self.is_activity_enabled

        name = self.name

        order = self.order.value

        owner_id = self.owner_id

        thumbnail_asset_id: Union[None, str]
        thumbnail_asset_id = self.thumbnail_asset_id

        updated_at = self.updated_at.isoformat()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "createdAt": created_at,
                "description": description,
                "id": id,
                "isActivityEnabled": is_activity_enabled,
                "name": name,
                "order": order,
                "ownerId": owner_id,
                "thumbnailAssetId": thumbnail_asset_id,
                "updatedAt": updated_at,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        created_at = isoparse(d.pop("createdAt"))

        description = d.pop("description")

        id = d.pop("id")

        is_activity_enabled = d.pop("isActivityEnabled")

        name = d.pop("name")

        order = AssetOrder(d.pop("order"))

        owner_id = d.pop("ownerId")

        def _parse_thumbnail_asset_id(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        thumbnail_asset_id = _parse_thumbnail_asset_id(d.pop("thumbnailAssetId"))

        updated_at = isoparse(d.pop("updatedAt"))

        sync_album_v1 = cls(
            created_at=created_at,
            description=description,
            id=id,
            is_activity_enabled=is_activity_enabled,
            name=name,
            order=order,
            owner_id=owner_id,
            thumbnail_asset_id=thumbnail_asset_id,
            updated_at=updated_at,
        )

        sync_album_v1.additional_properties = d
        return sync_album_v1

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.album_user_role import AlbumUserRole

if TYPE_CHECKING:
    from ..models.user_response_dto import UserResponseDto


T = TypeVar("T", bound="AlbumUserResponseDto")


@_attrs_define
class AlbumUserResponseDto:
    """
    Attributes:
        role (AlbumUserRole):
        user (UserResponseDto):
    """

    role: AlbumUserRole
    user: "UserResponseDto"
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        role = self.role.value

        user = self.user.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "role": role,
                "user": user,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.user_response_dto import UserResponseDto

        d = dict(src_dict)
        role = AlbumUserRole(d.pop("role"))

        user = UserResponseDto.from_dict(d.pop("user"))

        album_user_response_dto = cls(
            role=role,
            user=user,
        )

        album_user_response_dto.additional_properties = d
        return album_user_response_dto

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="SystemConfigSmtpTransportDto")


@_attrs_define
class SystemConfigSmtpTransportDto:
    """
    Attributes:
        host (str):
        ignore_cert (bool):
        password (str):
        port (float):
        username (str):
    """

    host: str
    ignore_cert: bool
    password: str
    port: float
    username: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        host = self.host

        ignore_cert = self.ignore_cert

        password = self.password

        port = self.port

        username = self.username

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "host": host,
                "ignoreCert": ignore_cert,
                "password": password,
                "port": port,
                "username": username,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        host = d.pop("host")

        ignore_cert = d.pop("ignoreCert")

        password = d.pop("password")

        port = d.pop("port")

        username = d.pop("username")

        system_config_smtp_transport_dto = cls(
            host=host,
            ignore_cert=ignore_cert,
            password=password,
            port=port,
            username=username,
        )

        system_config_smtp_transport_dto.additional_properties = d
        return system_config_smtp_transport_dto

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from http import HTTPStatus
from typing import Any, Optional, Union
from uuid import UUID

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.asset_response_dto import AssetResponseDto
from ...models.asset_type_enum import AssetTypeEnum
from ...models.asset_visibility import AssetVisibility
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    album_ids: Union[Unset, list[UUID]] = UNSET,
    city: Union[None, Unset, str] = UNSET,
    country: Union[None, Unset, str] = UNSET,
    created_after: Union[Unset, datetime.datetime] = UNSET,
    created_before: Union[Unset, datetime.datetime] = UNSET,
    device_id: Union[Unset, str] = UNSET,
    is_encoded: Union[Unset, bool] = UNSET,
    is_favorite: Union[Unset, bool] = UNSET,
    is_motion: Union[Unset, bool] = UNSET,
    is_not_in_album: Union[Unset, bool] = UNSET,
    is_offline: Union[Unset, bool] = UNSET,
    lens_model: Union[None, Unset, str] = UNSET,
    library_id: Union[None, UUID, Unset] = UNSET,
    make: Union[Unset, str] = UNSET,
    min_file_size: Union[Unset, int] = UNSET,
    model: Union[None, Unset, str] = UNSET,
    person_ids: Union[Unset, list[UUID]] = UNSET,
    rating: Union[Unset, float] = UNSET,
    size: Union[Unset, float] = UNSET,
    state: Union[None, Unset, str] = UNSET,
    tag_ids: Union[None, Unset, list[UUID]] = UNSET,
    taken_after: Union[Unset, datetime.datetime] = UNSET,
    taken_before: Union[Unset, datetime.datetime] = UNSET,
    trashed_after: Union[Unset, datetime.datetime] = UNSET,
    trashed_before: Union[Unset, datetime.datetime] = UNSET,
    type_: Union[Unset, AssetTypeEnum] = UNSET,
    updated_after: Union[Unset, datetime.datetime] = UNSET,
    updated_before: Union[Unset, datetime.datetime] = UNSET,
    visibility: Union[Unset, AssetVisibility] = UNSET,
    with_deleted: Union[Unset, bool] = UNSET,
    with_exif: Union[Unset, bool] = UNSET,
) -> dict[str, Any]:
    params: dict[str, Any] = {}

    json_album_ids: Union[Unset, list[str]] = UNSET
    if not isinstance(album_ids, Unset):
        json_album_ids = []
        for album_ids_item_data in album_ids:
            album_ids_item = str(album_ids_item_data)
            json_album_ids.append(album_ids_item)

    params["albumIds"] = json_album_ids

    json_city: Union[None, Unset, str]
    if isinstance(city, Unset):
        json_city = UNSET
    else:
        json_city = city
    params["city"] = json_city

    json_country: Union[None, Unset, str]
    if isinstance(country, Unset):
        json_country = UNSET
    else:
        json_country = country
    params["country"] = json_country

    json_created_after: Union[Unset, str] = UNSET
    if not isinstance(created_after, Unset):
        json_created_after = created_after.isoformat()
    params["createdAfter"] = json_created_after

    json_created_before: Union[Unset, str] = UNSET
    if not isinstance(created_before, Unset):
        json_created_before = created_before.isoformat()
    params["createdBefore"] = json_created_before

    params["deviceId"] = device_id

    params["isEncoded"] = is_encoded

    params["isFavorite"] = is_favorite

    params["isMotion"] = is_motion

    params["isNotInAlbum"] = is_not_in_album

    params["isOffline"] = is_offline

    json_lens_model: Union[None, Unset, str]
    if isinstance(lens_model, Unset):
        json_lens_model = UNSET
    else:
        json_lens_model = lens_model
    params["lensModel"] = json_lens_model

    json_library_id: Union[None, Unset, str]
    if isinstance(library_id, Unset):
        json_library_id = UNSET
    elif isinstance(library_id, UUID):
        json_library_id = str(library_id)
    else:
        json_library_id = library_id
    params["libraryId"] = json_library_id

    params["make"] = make

    params["minFileSize"] = min_file_size

    json_model: Union[None, Unset, str]
    if isinstance(model, Unset):
        json_model = UNSET
    else:
        json_model = model
    params["model"] = json_model

    json_person_ids: Union[Unset, list[str]] = UNSET
    if not isinstance(person_ids, Unset):
        json_person_ids = []
        for person_ids_item_data in person_ids:
            person_ids_item = str(person_ids_item_data)
            json_person_ids.append(person_ids_item)

    params["personIds"] = json_person_ids

    params["rating"] = rating

    params["size"] = size

    json_state: Union[None, Unset, str]
    if isinstance(state, Unset):
        json_state = UNSET
    else:
        json_state = state
    params["state"] = json_state

    json_tag_ids: Union[None, Unset, list[str]]
    if isinstance(tag_ids, Unset):
        json_tag_ids = UNSET
    elif isinstance(tag_ids, list):
        json_tag_ids = []
        for tag_ids_type_0_item_data in tag_ids:
            tag_ids_type_0_item = str(tag_ids_type_0_item_data)
            json_tag_ids.append(tag_ids_type_0_item)

    else:
        json_tag_ids = tag_ids
    params["tagIds"] = json_tag_ids

    json_taken_after: Union[Unset, str] = UNSET
    if not isinstance(taken_after, Unset):
        json_taken_after = taken_after.isoformat()
    params["takenAfter"] = json_taken_after

    json_taken_before: Union[Unset, str] = UNSET
    if not isinstance(taken_before, Unset):
        json_taken_before = taken_before.isoformat()
    params["takenBefore"] = json_taken_before

    json_trashed_after: Union[Unset, str] = UNSET
    if not isinstance(trashed_after, Unset):
        json_trashed_after = trashed_after.isoformat()
    params["trashedAfter"] = json_trashed_after

    json_trashed_before: Union[Unset, str] = UNSET
    if not isinstance(trashed_before, Unset):
        json_trashed_before = trashed_before.isoformat()
    params["trashedBefore"] = json_trashed_before

    json_type_: Union[Unset, str] = UNSET
    if not isinstance(type_, Unset):
        json_type_ = type_.value

    params["type"] = json_type_

    json_updated_after: Union[Unset, str] = UNSET
    if not isinstance(updated_after, Unset):
        json_updated_after = updated_after.isoformat()
    params["updatedAfter"] = json_updated_after

    json_updated_before: Union[Unset, str] = UNSET
    if not isinstance(updated_before, Unset):
        json_updated_before = updated_before.isoformat()
    params["updatedBefore"] = json_updated_before

    json_visibility: Union[Unset, str] = UNSET
    if not isinstance(visibility, Unset):
        json_visibility = visibility.value

    params["visibility"] = json_visibility

    params["withDeleted"] = with_deleted

    params["withExif"] = with_exif

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/search/large-assets",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[list["AssetResponseDto"]]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = AssetResponseDto.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[list["AssetResponseDto"]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    album_ids: Union[Unset, list[UUID]] = UNSET,
    city: Union[None, Unset, str] = UNSET,
    country: Union[None, Unset, str] = UNSET,
    created_after: Union[Unset, datetime.datetime] = UNSET,
    created_before: Union[Unset, datetime.datetime] = UNSET,
    device_id: Union[Unset, str] = UNSET,
    is_encoded: Union[Unset, bool] = UNSET,
    is_favorite: Union[Unset, bool] = UNSET,
    is_motion: Union[Unset, bool] = UNSET,
    is_not_in_album: Union[Unset, bool] = UNSET,
    is_offline: Union[Unset, bool] = UNSET,
    lens_model: Union[None, Unset, str] = UNSET,
    library_id: Union[None, UUID, Unset] = UNSET,
    make: Union[Unset, str] = UNSET,
    min_file_size: Union[Unset, int] = UNSET,
    model: Union[None, Unset, str] = UNSET,
    person_ids: Union[Unset, list[UUID]] = UNSET,
    rating: Union[Unset, float] = UNSET,
    size: Union[Unset, float] = UNSET,
    state: Union[None, Unset, str] = UNSET,
    tag_ids: Union[None, Unset, list[UUID]] = UNSET,
    taken_after: Union[Unset, datetime.datetime] = UNSET,
    taken_before: Union[Unset, datetime.datetime] = UNSET,
    trashed_after: Union[Unset, datetime.datetime] = UNSET,
    trashed_before: Union[Unset, datetime.datetime] = UNSET,
    type_: Union[Unset, AssetTypeEnum] = UNSET,
    updated_after: Union[Unset, datetime.datetime] = UNSET,
    updated_before: Union[Unset, datetime.datetime] = UNSET,
    visibility: Union[Unset, AssetVisibility] = UNSET,
    with_deleted: Union[Unset, bool] = UNSET,
    with_exif: Union[Unset, bool] = UNSET,
) -> Response[list["AssetResponseDto"]]:
    """This endpoint requires the `asset.read` permission.

    Args:
        album_ids (Union[Unset, list[UUID]]):
        city (Union[None, Unset, str]):
        country (Union[None, Unset, str]):
        created_after (Union[Unset, datetime.datetime]):
        created_before (Union[Unset, datetime.datetime]):
        device_id (Union[Unset, str]):
        is_encoded (Union[Unset, bool]):
        is_favorite (Union[Unset, bool]):
        is_motion (Union[Unset, bool]):
        is_not_in_album (Union[Unset, bool]):
        is_offline (Union[Unset, bool]):
        lens_model (Union[None, Unset, str]):
        library_id (Union[None, UUID, Unset]):
        make (Union[Unset, str]):
        min_file_size (Union[Unset, int]):
        model (Union[None, Unset, str]):
        person_ids (Union[Unset, list[UUID]]):
        rating (Union[Unset, float]):
        size (Union[Unset, float]):
        state (Union[None, Unset, str]):
        tag_ids (Union[None, Unset, list[UUID]]):
        taken_after (Union[Unset, datetime.datetime]):
        taken_before (Union[Unset, datetime.datetime]):
        trashed_after (Union[Unset, datetime.datetime]):
        trashed_before (Union[Unset, datetime.datetime]):
        type_ (Union[Unset, AssetTypeEnum]):
        updated_after (Union[Unset, datetime.datetime]):
        updated_before (Union[Unset, datetime.datetime]):
        visibility (Union[Unset, AssetVisibility]):
        with_deleted (Union[Unset, bool]):
        with_exif (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[list['AssetResponseDto']]
    """

    kwargs = _get_kwargs(
        album_ids=album_ids,
        city=city,
        country=country,
        created_after=created_after,
        created_before=created_before,
        device_id=device_id,
        is_encoded=is_encoded,
        is_favorite=is_favorite,
        is_motion=is_motion,
        is_not_in_album=is_not_in_album,
        is_offline=is_offline,
        lens_model=lens_model,
        library_id=library_id,
        make=make,
        min_file_size=min_file_size,
        model=model,
        person_ids=person_ids,
        rating=rating,
        size=size,
        state=state,
        tag_ids=tag_ids,
        taken_after=taken_after,
        taken_before=taken_before,
        trashed_after=trashed_after,
        trashed_before=trashed_before,
        type_=type_,
        updated_after=updated_after,
        updated_before=updated_before,
        visibility=visibility,
        with_deleted=with_deleted,
        with_exif=with_exif,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    album_ids: Union[Unset, list[UUID]] = UNSET,
    city: Union[None, Unset, str] = UNSET,
    country: Union[None, Unset, str] = UNSET,
    created_after: Union[Unset, datetime.datetime] = UNSET,
    created_before: Union[Unset, datetime.datetime] = UNSET,
    device_id: Union[Unset, str] = UNSET,
    is_encoded: Union[Unset, bool] = UNSET,
    is_favorite: Union[Unset, bool] = UNSET,
    is_motion: Union[Unset, bool] = UNSET,
    is_not_in_album: Union[Unset, bool] = UNSET,
    is_offline: Union[Unset, bool] = UNSET,
    lens_model: Union[None, Unset, str] = UNSET,
    library_id: Union[None, UUID, Unset] = UNSET,
    make: Union[Unset, str] = UNSET,
    min_file_size: Union[Unset, int] = UNSET,
    model: Union[None, Unset, str] = UNSET,
    person_ids: Union[Unset, list[UUID]] = UNSET,
    rating: Union[Unset, float] = UNSET,
    size: Union[Unset, float] = UNSET,
    state: Union[None, Unset, str] = UNSET,
    tag_ids: Union[None, Unset, list[UUID]] = UNSET,
    taken_after: Union[Unset, datetime.datetime] = UNSET,
    taken_before: Union[Unset, datetime.datetime] = UNSET,
    trashed_after: Union[Unset, datetime.datetime] = UNSET,
    trashed_before: Union[Unset, datetime.datetime] = UNSET,
    type_: Union[Unset, AssetTypeEnum] = UNSET,
    updated_after: Union[Unset, datetime.datetime] = UNSET,
    updated_before: Union[Unset, datetime.datetime] = UNSET,
    visibility: Union[Unset, AssetVisibility] = UNSET,
    with_deleted: Union[Unset, bool] = UNSET,
    with_exif: Union[Unset, bool] = UNSET,
) -> Optional[list["AssetResponseDto"]]:
    """This endpoint requires the `asset.read` permission.

    Args:
        album_ids (Union[Unset, list[UUID]]):
        city (Union[None, Unset, str]):
        country (Union[None, Unset, str]):
        created_after (Union[Unset, datetime.datetime]):
        created_before (Union[Unset, datetime.datetime]):
        device_id (Union[Unset, str]):
        is_encoded (Union[Unset, bool]):
        is_favorite (Union[Unset, bool]):
        is_motion (Union[Unset, bool]):
        is_not_in_album (Union[Unset, bool]):
        is_offline (Union[Unset, bool]):
        lens_model (Union[None, Unset, str]):
        library_id (Union[None, UUID, Unset]):
        make (Union[Unset, str]):
        min_file_size (Union[Unset, int]):
        model (Union[None, Unset, str]):
        person_ids (Union[Unset, list[UUID]]):
        rating (Union[Unset, float]):
        size (Union[Unset, float]):
        state (Union[None, Unset, str]):
        tag_ids (Union[None, Unset, list[UUID]]):
        taken_after (Union[Unset, datetime.datetime]):
        taken_before (Union[Unset, datetime.datetime]):
        trashed_after (Union[Unset, datetime.datetime]):
        trashed_before (Union[Unset, datetime.datetime]):
        type_ (Union[Unset, AssetTypeEnum]):
        updated_after (Union[Unset, datetime.datetime]):
        updated_before (Union[Unset, datetime.datetime]):
        visibility (Union[Unset, AssetVisibility]):
        with_deleted (Union[Unset, bool]):
        with_exif (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        list['AssetResponseDto']
    """

    return sync_detailed(
        client=client,
        album_ids=album_ids,
        city=city,
        country=country,
        created_after=created_after,
        created_before=created_before,
        device_id=device_id,
        is_encoded=is_encoded,
        is_favorite=is_favorite,
        is_motion=is_motion,
        is_not_in_album=is_not_in_album,
        is_offline=is_offline,
        lens_model=lens_model,
        library_id=library_id,
        make=make,
        min_file_size=min_file_size,
        model=model,
        person_ids=person_ids,
        rating=rating,
        size=size,
        state=state,
        tag_ids=tag_ids,
        taken_after=taken_after,
        taken_before=taken_before,
        trashed_after=trashed_after,
        trashed_before=trashed_before,
        type_=type_,
        updated_after=updated_after,
        updated_before=updated_before,
        visibility=visibility,
        with_deleted=with_deleted,
        with_exif=with_exif,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    album_ids: Union[Unset, list[UUID]] = UNSET,
    city: Union[None, Unset, str] = UNSET,
    country: Union[None, Unset, str] = UNSET,
    created_after: Union[Unset, datetime.datetime] = UNSET,
    created_before: Union[Unset, datetime.datetime] = UNSET,
    device_id: Union[Unset, str] = UNSET,
    is_encoded: Union[Unset, bool] = UNSET,
    is_favorite: Union[Unset, bool] = UNSET,
    is_motion: Union[Unset, bool] = UNSET,
    is_not_in_album: Union[Unset, bool] = UNSET,
    is_offline: Union[Unset, bool] = UNSET,
    lens_model: Union[None, Unset, str] = UNSET,
    library_id: Union[None, UUID, Unset] = UNSET,
    make: Union[Unset, str] = UNSET,
    min_file_size: Union[Unset, int] = UNSET,
    model: Union[None, Unset, str] = UNSET,
    person_ids: Union[Unset, list[UUID]] = UNSET,
    rating: Union[Unset, float] = UNSET,
    size: Union[Unset, float] = UNSET,
    state: Union[None, Unset, str] = UNSET,
    tag_ids: Union[None, Unset, list[UUID]] = UNSET,
    taken_after: Union[Unset, datetime.datetime] = UNSET,
    taken_before: Union[Unset, datetime.datetime] = UNSET,
    trashed_after: Union[Unset, datetime.datetime] = UNSET,
    trashed_before: Union[Unset, datetime.datetime] = UNSET,
    type_: Union[Unset, AssetTypeEnum] = UNSET,
    updated_after: Union[Unset, datetime.datetime] = UNSET,
    updated_before: Union[Unset, datetime.datetime] = UNSET,
    visibility: Union[Unset, AssetVisibility] = UNSET,
    with_deleted: Union[Unset, bool] = UNSET,
    with_exif: Union[Unset, bool] = UNSET,
) -> Response[list["AssetResponseDto"]]:
    """This endpoint requires the `asset.read` permission.

    Args:
        album_ids (Union[Unset, list[UUID]]):
        city (Union[None, Unset, str]):
        country (Union[None, Unset, str]):
        created_after (Union[Unset, datetime.datetime]):
        created_before (Union[Unset, datetime.datetime]):
        device_id (Union[Unset, str]):
        is_encoded (Union[Unset, bool]):
        is_favorite (Union[Unset, bool]):
        is_motion (Union[Unset, bool]):
        is_not_in_album (Union[Unset, bool]):
        is_offline (Union[Unset, bool]):
        lens_model (Union[None, Unset, str]):
        library_id (Union[None, UUID, Unset]):
        make (Union[Unset, str]):
        min_file_size (Union[Unset, int]):
        model (Union[None, Unset, str]):
        person_ids (Union[Unset, list[UUID]]):
        rating (Union[Unset, float]):
        size (Union[Unset, float]):
        state (Union[None, Unset, str]):
        tag_ids (Union[None, Unset, list[UUID]]):
        taken_after (Union[Unset, datetime.datetime]):
        taken_before (Union[Unset, datetime.datetime]):
        trashed_after (Union[Unset, datetime.datetime]):
        trashed_before (Union[Unset, datetime.datetime]):
        type_ (Union[Unset, AssetTypeEnum]):
        updated_after (Union[Unset, datetime.datetime]):
        updated_before (Union[Unset, datetime.datetime]):
        visibility (Union[Unset, AssetVisibility]):
        with_deleted (Union[Unset, bool]):
        with_exif (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[list['AssetResponseDto']]
    """

    kwargs = _get_kwargs(
        album_ids=album_ids,
        city=city,
        country=country,
        created_after=created_after,
        created_before=created_before,
        device_id=device_id,
        is_encoded=is_encoded,
        is_favorite=is_favorite,
        is_motion=is_motion,
        is_not_in_album=is_not_in_album,
        is_offline=is_offline,
        lens_model=lens_model,
        library_id=library_id,
        make=make,
        min_file_size=min_file_size,
        model=model,
        person_ids=person_ids,
        rating=rating,
        size=size,
        state=state,
        tag_ids=tag_ids,
        taken_after=taken_after,
        taken_before=taken_before,
        trashed_after=trashed_after,
        trashed_before=trashed_before,
        type_=type_,
        updated_after=updated_after,
        updated_before=updated_before,
        visibility=visibility,
        with_deleted=with_deleted,
        with_exif=with_exif,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    album_ids: Union[Unset, list[UUID]] = UNSET,
    city: Union[None, Unset, str] = UNSET,
    country: Union[None, Unset, str] = UNSET,
    created_after: Union[Unset, datetime.datetime] = UNSET,
    created_before: Union[Unset, datetime.datetime] = UNSET,
    device_id: Union[Unset, str] = UNSET,
    is_encoded: Union[Unset, bool] = UNSET,
    is_favorite: Union[Unset, bool] = UNSET,
    is_motion: Union[Unset, bool] = UNSET,
    is_not_in_album: Union[Unset, bool] = UNSET,
    is_offline: Union[Unset, bool] = UNSET,
    lens_model: Union[None, Unset, str] = UNSET,
    library_id: Union[None, UUID, Unset] = UNSET,
    make: Union[Unset, str] = UNSET,
    min_file_size: Union[Unset, int] = UNSET,
    model: Union[None, Unset, str] = UNSET,
    person_ids: Union[Unset, list[UUID]] = UNSET,
    rating: Union[Unset, float] = UNSET,
    size: Union[Unset, float] = UNSET,
    state: Union[None, Unset, str] = UNSET,
    tag_ids: Union[None, Unset, list[UUID]] = UNSET,
    taken_after: Union[Unset, datetime.datetime] = UNSET,
    taken_before: Union[Unset, datetime.datetime] = UNSET,
    trashed_after: Union[Unset, datetime.datetime] = UNSET,
    trashed_before: Union[Unset, datetime.datetime] = UNSET,
    type_: Union[Unset, AssetTypeEnum] = UNSET,
    updated_after: Union[Unset, datetime.datetime] = UNSET,
    updated_before: Union[Unset, datetime.datetime] = UNSET,
    visibility: Union[Unset, AssetVisibility] = UNSET,
    with_deleted: Union[Unset, bool] = UNSET,
    with_exif: Union[Unset, bool] = UNSET,
) -> Optional[list["AssetResponseDto"]]:
    """This endpoint requires the `asset.read` permission.

    Args:
        album_ids (Union[Unset, list[UUID]]):
        city (Union[None, Unset, str]):
        country (Union[None, Unset, str]):
        created_after (Union[Unset, datetime.datetime]):
        created_before (Union[Unset, datetime.datetime]):
        device_id (Union[Unset, str]):
        is_encoded (Union[Unset, bool]):
        is_favorite (Union[Unset, bool]):
        is_motion (Union[Unset, bool]):
        is_not_in_album (Union[Unset, bool]):
        is_offline (Union[Unset, bool]):
        lens_model (Union[None, Unset, str]):
        library_id (Union[None, UUID, Unset]):
        make (Union[Unset, str]):
        min_file_size (Union[Unset, int]):
        model (Union[None, Unset, str]):
        person_ids (Union[Unset, list[UUID]]):
        rating (Union[Unset, float]):
        size (Union[Unset, float]):
        state (Union[None, Unset, str]):
        tag_ids (Union[None, Unset, list[UUID]]):
        taken_after (Union[Unset, datetime.datetime]):
        taken_before (Union[Unset, datetime.datetime]):
        trashed_after (Union[Unset, datetime.datetime]):
        trashed_before (Union[Unset, datetime.datetime]):
        type_ (Union[Unset, AssetTypeEnum]):
        updated_after (Union[Unset, datetime.datetime]):
        updated_before (Union[Unset, datetime.datetime]):
        visibility (Union[Unset, AssetVisibility]):
        with_deleted (Union[Unset, bool]):
        with_exif (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        list['AssetResponseDto']
    """

    return (
        await asyncio_detailed(
            client=client,
            album_ids=album_ids,
            city=city,
            country=country,
            created_after=created_after,
            created_before=created_before,
            device_id=device_id,
            is_encoded=is_encoded,
            is_favorite=is_favorite,
            is_motion=is_motion,
            is_not_in_album=is_not_in_album,
            is_offline=is_offline,
            lens_model=lens_model,
            library_id=library_id,
            make=make,
            min_file_size=min_file_size,
            model=model,
            person_ids=person_ids,
            rating=rating,
            size=size,
            state=state,
            tag_ids=tag_ids,
            taken_after=taken_after,
            taken_before=taken_before,
            trashed_after=trashed_after,
            trashed_before=trashed_before,
            type_=type_,
            updated_after=updated_after,
            updated_before=updated_before,
            visibility=visibility,
            with_deleted=with_deleted,
            with_exif=with_exif,
        )
    ).parsed



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="PlacesResponseDto")


@_attrs_define
class PlacesResponseDto:
    """
    Attributes:
        latitude (float):
        longitude (float):
        name (str):
        admin1name (Union[Unset, str]):
        admin2name (Union[Unset, str]):
    """

    latitude: float
    longitude: float
    name: str
    admin1name: Union[Unset, str] = UNSET
    admin2name: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        latitude = self.latitude

        longitude = self.longitude

        name = self.name

        admin1name = self.admin1name

        admin2name = self.admin2name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "latitude": latitude,
                "longitude": longitude,
                "name": name,
            }
        )
        if admin1name is not UNSET:
            field_dict["admin1name"] = admin1name
        if admin2name is not UNSET:
            field_dict["admin2name"] = admin2name

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        latitude = d.pop("latitude")

        longitude = d.pop("longitude")

        name = d.pop("name")

        admin1name = d.pop("admin1name", UNSET)

        admin2name = d.pop("admin2name", UNSET)

        places_response_dto = cls(
            latitude=latitude,
            longitude=longitude,
            name=name,
            admin1name=admin1name,
            admin2name=admin2name,
        )

        places_response_dto.additional_properties = d
        return places_response_dto

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Database module for immichporter."""

from .commands import (
    init,
    show_albums,
    show_users,
    edit_users,
    show_stats,
    drop_command,
)

__all__ = [
    "init",
    "show_albums",
    "show_users",
    "edit_users",
    "show_stats",
    "drop_command",
]



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.album_user_create_dto import AlbumUserCreateDto


T = TypeVar("T", bound="CreateAlbumDto")


@_attrs_define
class CreateAlbumDto:
    """
    Attributes:
        album_name (str):
        album_users (Union[Unset, list['AlbumUserCreateDto']]):
        asset_ids (Union[Unset, list[UUID]]):
        description (Union[Unset, str]):
    """

    album_name: str
    album_users: Union[Unset, list["AlbumUserCreateDto"]] = UNSET
    asset_ids: Union[Unset, list[UUID]] = UNSET
    description: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        album_name = self.album_name

        album_users: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.album_users, Unset):
            album_users = []
            for album_users_item_data in self.album_users:
                album_users_item = album_users_item_data.to_dict()
                album_users.append(album_users_item)

        asset_ids: Union[Unset, list[str]] = UNSET
        if not isinstance(self.asset_ids, Unset):
            asset_ids = []
            for asset_ids_item_data in self.asset_ids:
                asset_ids_item = str(asset_ids_item_data)
                asset_ids.append(asset_ids_item)

        description = self.description

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "albumName": album_name,
            }
        )
        if album_users is not UNSET:
            field_dict["albumUsers"] = album_users
        if asset_ids is not UNSET:
            field_dict["assetIds"] = asset_ids
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.album_user_create_dto import AlbumUserCreateDto

        d = dict(src_dict)
        album_name = d.pop("albumName")

        album_users = []
        _album_users = d.pop("albumUsers", UNSET)
        for album_users_item_data in _album_users or []:
            album_users_item = AlbumUserCreateDto.from_dict(album_users_item_data)

            album_users.append(album_users_item)

        asset_ids = []
        _asset_ids = d.pop("assetIds", UNSET)
        for asset_ids_item_data in _asset_ids or []:
            asset_ids_item = UUID(asset_ids_item_data)

            asset_ids.append(asset_ids_item)

        description = d.pop("description", UNSET)

        create_album_dto = cls(
            album_name=album_name,
            album_users=album_users,
            asset_ids=asset_ids,
            description=description,
        )

        create_album_dto.additional_properties = d
        return create_album_dto

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="TimeBucketsResponseDto")


@_attrs_define
class TimeBucketsResponseDto:
    """
    Attributes:
        count (int): Number of assets in this time bucket Example: 42.
        time_bucket (str): Time bucket identifier in YYYY-MM-DD format representing the start of the time period
            Example: 2024-01-01.
    """

    count: int
    time_bucket: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        count = self.count

        time_bucket = self.time_bucket

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "count": count,
                "timeBucket": time_bucket,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        count = d.pop("count")

        time_bucket = d.pop("timeBucket")

        time_buckets_response_dto = cls(
            count=count,
            time_bucket=time_bucket,
        )

        time_buckets_response_dto.additional_properties = d
        return time_buckets_response_dto

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Optional, Union
from uuid import UUID

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.activity_response_dto import ActivityResponseDto
from ...models.reaction_level import ReactionLevel
from ...models.reaction_type import ReactionType
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    album_id: UUID,
    asset_id: Union[Unset, UUID] = UNSET,
    level: Union[Unset, ReactionLevel] = UNSET,
    type_: Union[Unset, ReactionType] = UNSET,
    user_id: Union[Unset, UUID] = UNSET,
) -> dict[str, Any]:
    params: dict[str, Any] = {}

    json_album_id = str(album_id)
    params["albumId"] = json_album_id

    json_asset_id: Union[Unset, str] = UNSET
    if not isinstance(asset_id, Unset):
        json_asset_id = str(asset_id)
    params["assetId"] = json_asset_id

    json_level: Union[Unset, str] = UNSET
    if not isinstance(level, Unset):
        json_level = level.value

    params["level"] = json_level

    json_type_: Union[Unset, str] = UNSET
    if not isinstance(type_, Unset):
        json_type_ = type_.value

    params["type"] = json_type_

    json_user_id: Union[Unset, str] = UNSET
    if not isinstance(user_id, Unset):
        json_user_id = str(user_id)
    params["userId"] = json_user_id

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/activities",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[list["ActivityResponseDto"]]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = ActivityResponseDto.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[list["ActivityResponseDto"]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    album_id: UUID,
    asset_id: Union[Unset, UUID] = UNSET,
    level: Union[Unset, ReactionLevel] = UNSET,
    type_: Union[Unset, ReactionType] = UNSET,
    user_id: Union[Unset, UUID] = UNSET,
) -> Response[list["ActivityResponseDto"]]:
    """This endpoint requires the `activity.read` permission.

    Args:
        album_id (UUID):
        asset_id (Union[Unset, UUID]):
        level (Union[Unset, ReactionLevel]):
        type_ (Union[Unset, ReactionType]):
        user_id (Union[Unset, UUID]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[list['ActivityResponseDto']]
    """

    kwargs = _get_kwargs(
        album_id=album_id,
        asset_id=asset_id,
        level=level,
        type_=type_,
        user_id=user_id,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    album_id: UUID,
    asset_id: Union[Unset, UUID] = UNSET,
    level: Union[Unset, ReactionLevel] = UNSET,
    type_: Union[Unset, ReactionType] = UNSET,
    user_id: Union[Unset, UUID] = UNSET,
) -> Optional[list["ActivityResponseDto"]]:
    """This endpoint requires the `activity.read` permission.

    Args:
        album_id (UUID):
        asset_id (Union[Unset, UUID]):
        level (Union[Unset, ReactionLevel]):
        type_ (Union[Unset, ReactionType]):
        user_id (Union[Unset, UUID]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        list['ActivityResponseDto']
    """

    return sync_detailed(
        client=client,
        album_id=album_id,
        asset_id=asset_id,
        level=level,
        type_=type_,
        user_id=user_id,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    album_id: UUID,
    asset_id: Union[Unset, UUID] = UNSET,
    level: Union[Unset, ReactionLevel] = UNSET,
    type_: Union[Unset, ReactionType] = UNSET,
    user_id: Union[Unset, UUID] = UNSET,
) -> Response[list["ActivityResponseDto"]]:
    """This endpoint requires the `activity.read` permission.

    Args:
        album_id (UUID):
        asset_id (Union[Unset, UUID]):
        level (Union[Unset, ReactionLevel]):
        type_ (Union[Unset, ReactionType]):
        user_id (Union[Unset, UUID]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[list['ActivityResponseDto']]
    """

    kwargs = _get_kwargs(
        album_id=album_id,
        asset_id=asset_id,
        level=level,
        type_=type_,
        user_id=user_id,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    album_id: UUID,
    asset_id: Union[Unset, UUID] = UNSET,
    level: Union[Unset, ReactionLevel] = UNSET,
    type_: Union[Unset, ReactionType] = UNSET,
    user_id: Union[Unset, UUID] = UNSET,
) -> Optional[list["ActivityResponseDto"]]:
    """This endpoint requires the `activity.read` permission.

    Args:
        album_id (UUID):
        asset_id (Union[Unset, UUID]):
        level (Union[Unset, ReactionLevel]):
        type_ (Union[Unset, ReactionType]):
        user_id (Union[Unset, UUID]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        list['ActivityResponseDto']
    """

    return (
        await asyncio_detailed(
            client=client,
            album_id=album_id,
            asset_id=asset_id,
            level=level,
            type_=type_,
            user_id=user_id,
        )
    ).parsed



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.system_config_smtp_transport_dto import SystemConfigSmtpTransportDto


T = TypeVar("T", bound="SystemConfigSmtpDto")


@_attrs_define
class SystemConfigSmtpDto:
    """
    Attributes:
        enabled (bool):
        from_ (str):
        reply_to (str):
        transport (SystemConfigSmtpTransportDto):
    """

    enabled: bool
    from_: str
    reply_to: str
    transport: "SystemConfigSmtpTransportDto"
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        enabled = self.enabled

        from_ = self.from_

        reply_to = self.reply_to

        transport = self.transport.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "enabled": enabled,
                "from": from_,
                "replyTo": reply_to,
                "transport": transport,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.system_config_smtp_transport_dto import (
            SystemConfigSmtpTransportDto,
        )

        d = dict(src_dict)
        enabled = d.pop("enabled")

        from_ = d.pop("from")

        reply_to = d.pop("replyTo")

        transport = SystemConfigSmtpTransportDto.from_dict(d.pop("transport"))

        system_config_smtp_dto = cls(
            enabled=enabled,
            from_=from_,
            reply_to=reply_to,
            transport=transport,
        )

        system_config_smtp_dto.additional_properties = d
        return system_config_smtp_dto

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Pydantic models for Immich API responses."""

from datetime import datetime
from enum import Enum
from typing import List, Optional, Dict, Any
from pydantic import BaseModel, Field


class AssetType(str, Enum):
    IMAGE = "IMAGE"
    VIDEO = "VIDEO"
    AUDIO = "AUDIO"
    OTHER = "OTHER"


class AssetResponse(BaseModel):
    """Represents an asset (photo/video) in Immich."""

    id: str
    device_asset_id: str = Field(..., alias="deviceAssetId")
    owner_id: str = Field(..., alias="ownerId")
    device_id: str = Field(..., alias="deviceId")
    type: AssetType
    original_path: str = Field(..., alias="originalPath")
    original_file_name: str = Field(..., alias="originalFileName")
    file_created_at: datetime = Field(..., alias="fileCreatedAt")
    file_modified_at: datetime = Field(..., alias="fileModifiedAt")
    updated_at: datetime = Field(..., alias="updatedAt")
    is_favorite: bool = Field(..., alias="isFavorite")
    is_archived: bool = Field(..., alias="isArchived")
    duration: Optional[str] = None
    exif_info: Optional[Dict[str, Any]] = Field(None, alias="exifInfo")


class AlbumResponse(BaseModel):
    """Represents an album in Immich."""

    id: str
    album_name: str = Field(..., alias="albumName")
    description: Optional[str] = None
    album_thumbnail_asset_id: Optional[str] = Field(None, alias="albumThumbnailAssetId")
    shared: bool
    asset_count: int = Field(..., alias="assetCount")
    assets: List[AssetResponse] = []
    created_at: datetime = Field(..., alias="createdAt")
    updated_at: datetime = Field(..., alias="updatedAt")
    album_thumbnail_asset: Optional[AssetResponse] = Field(
        None, alias="albumThumbnailAsset"
    )


class AlbumListResponse(BaseModel):
    """Response model for listing albums."""

    albums: List[AlbumResponse]
    count: int



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.bulk_id_error_reason import BulkIdErrorReason
from ..types import UNSET, Unset

T = TypeVar("T", bound="AlbumsAddAssetsResponseDto")


@_attrs_define
class AlbumsAddAssetsResponseDto:
    """
    Attributes:
        success (bool):
        error (Union[Unset, BulkIdErrorReason]):
    """

    success: bool
    error: Union[Unset, BulkIdErrorReason] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        success = self.success

        error: Union[Unset, str] = UNSET
        if not isinstance(self.error, Unset):
            error = self.error.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "success": success,
            }
        )
        if error is not UNSET:
            field_dict["error"] = error

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        success = d.pop("success")

        _error = d.pop("error", UNSET)
        error: Union[Unset, BulkIdErrorReason]
        if isinstance(_error, Unset):
            error = UNSET
        else:
            error = BulkIdErrorReason(_error)

        albums_add_assets_response_dto = cls(
            success=success,
            error=error,
        )

        albums_add_assets_response_dto.additional_properties = d
        return albums_add_assets_response_dto

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="SyncAssetFaceV1")


@_attrs_define
class SyncAssetFaceV1:
    """
    Attributes:
        asset_id (str):
        bounding_box_x1 (int):
        bounding_box_x2 (int):
        bounding_box_y1 (int):
        bounding_box_y2 (int):
        id (str):
        image_height (int):
        image_width (int):
        person_id (Union[None, str]):
        source_type (str):
    """

    asset_id: str
    bounding_box_x1: int
    bounding_box_x2: int
    bounding_box_y1: int
    bounding_box_y2: int
    id: str
    image_height: int
    image_width: int
    person_id: Union[None, str]
    source_type: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        asset_id = self.asset_id

        bounding_box_x1 = self.bounding_box_x1

        bounding_box_x2 = self.bounding_box_x2

        bounding_box_y1 = self.bounding_box_y1

        bounding_box_y2 = self.bounding_box_y2

        id = self.id

        image_height = self.image_height

        image_width = self.image_width

        person_id: Union[None, str]
        person_id = self.person_id

        source_type = self.source_type

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "assetId": asset_id,
                "boundingBoxX1": bounding_box_x1,
                "boundingBoxX2": bounding_box_x2,
                "boundingBoxY1": bounding_box_y1,
                "boundingBoxY2": bounding_box_y2,
                "id": id,
                "imageHeight": image_height,
                "imageWidth": image_width,
                "personId": person_id,
                "sourceType": source_type,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        asset_id = d.pop("assetId")

        bounding_box_x1 = d.pop("boundingBoxX1")

        bounding_box_x2 = d.pop("boundingBoxX2")

        bounding_box_y1 = d.pop("boundingBoxY1")

        bounding_box_y2 = d.pop("boundingBoxY2")

        id = d.pop("id")

        image_height = d.pop("imageHeight")

        image_width = d.pop("imageWidth")

        def _parse_person_id(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        person_id = _parse_person_id(d.pop("personId"))

        source_type = d.pop("sourceType")

        sync_asset_face_v1 = cls(
            asset_id=asset_id,
            bounding_box_x1=bounding_box_x1,
            bounding_box_x2=bounding_box_x2,
            bounding_box_y1=bounding_box_y1,
            bounding_box_y2=bounding_box_y2,
            id=id,
            image_height=image_height,
            image_width=image_width,
            person_id=person_id,
            source_type=source_type,
        )

        sync_asset_face_v1.additional_properties = d
        return sync_asset_face_v1

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.clip_config import CLIPConfig
    from ..models.duplicate_detection_config import DuplicateDetectionConfig
    from ..models.facial_recognition_config import FacialRecognitionConfig
    from ..models.machine_learning_availability_checks_dto import (
        MachineLearningAvailabilityChecksDto,
    )


T = TypeVar("T", bound="SystemConfigMachineLearningDto")


@_attrs_define
class SystemConfigMachineLearningDto:
    """
    Attributes:
        availability_checks (MachineLearningAvailabilityChecksDto):
        clip (CLIPConfig):
        duplicate_detection (DuplicateDetectionConfig):
        enabled (bool):
        facial_recognition (FacialRecognitionConfig):
        urls (list[str]):
    """

    availability_checks: "MachineLearningAvailabilityChecksDto"
    clip: "CLIPConfig"
    duplicate_detection: "DuplicateDetectionConfig"
    enabled: bool
    facial_recognition: "FacialRecognitionConfig"
    urls: list[str]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        availability_checks = self.availability_checks.to_dict()

        clip = self.clip.to_dict()

        duplicate_detection = self.duplicate_detection.to_dict()

        enabled = self.enabled

        facial_recognition = self.facial_recognition.to_dict()

        urls = self.urls

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "availabilityChecks": availability_checks,
                "clip": clip,
                "duplicateDetection": duplicate_detection,
                "enabled": enabled,
                "facialRecognition": facial_recognition,
                "urls": urls,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.clip_config import CLIPConfig
        from ..models.duplicate_detection_config import DuplicateDetectionConfig
        from ..models.facial_recognition_config import FacialRecognitionConfig
        from ..models.machine_learning_availability_checks_dto import (
            MachineLearningAvailabilityChecksDto,
        )

        d = dict(src_dict)
        availability_checks = MachineLearningAvailabilityChecksDto.from_dict(
            d.pop("availabilityChecks")
        )

        clip = CLIPConfig.from_dict(d.pop("clip"))

        duplicate_detection = DuplicateDetectionConfig.from_dict(
            d.pop("duplicateDetection")
        )

        enabled = d.pop("enabled")

        facial_recognition = FacialRecognitionConfig.from_dict(
            d.pop("facialRecognition")
        )

        urls = cast(list[str], d.pop("urls"))

        system_config_machine_learning_dto = cls(
            availability_checks=availability_checks,
            clip=clip,
            duplicate_detection=duplicate_detection,
            enabled=enabled,
            facial_recognition=facial_recognition,
            urls=urls,
        )

        system_config_machine_learning_dto.additional_properties = d
        return system_config_machine_learning_dto

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Schemas for album and photos."""

from dataclasses import dataclass
from datetime import datetime
from typing import List, Optional


@dataclass
class AlbumInfo:
    """Information about a Google Photos album.

    Attributes:
        title: The title of the album
        items: Total number of items in the album
        shared: Whether the album is shared
        url: URL of the album
        album_id: Internal database ID of the album
        processed_items: Number of items that have been processed
        created_at: When the album was created in the database
        all_photos_saved: Whether all photos in the album have been saved
    """

    title: str
    items: int
    shared: bool
    url: str
    album_id: int | None = None
    processed_items: int = 0
    created_at: str | None = None
    all_photos_saved: bool = False


@dataclass
class PictureInfo:
    """Information about a Google Photos picture."""

    filename: str
    date_taken: Optional[datetime]
    user: Optional[str]
    source_id: str
    user_id: int | None = None
    saved_to_your_photos: bool = False


@dataclass
class ProcessingResult:
    """Data class for overall processing results."""

    total_albums: int
    total_pictures: int
    albums_processed: List[AlbumInfo]
    errors: List[str]


@dataclass
class ProcessingResult_error:
    """Data class for error information during processing."""

    error: str
    album_title: str = ""
    photo_url: str = ""



"""Database CLI commands."""

import click
import math
import json
from rich.console import Console
from rich.table import Table
from rich.text import Text
from sqlalchemy import text
from sqlalchemy.orm import Session
from sqlalchemy.exc import SQLAlchemyError
from immichporter.models import User, Album, Photo, Error
from immichporter.database import Base, SessionLocal
from immichporter.database import (
    get_db_session,
    get_albums_from_db,
    get_users_from_db,
    get_database_stats,
    init_database,
)
from immichporter.utils import sanitize_for_email, format_csv_value
from immichporter.commands import logging_options
from ..gphotos.utils import traceback
from loguru import logger


cli_db = click.Group(
    "db", help="Show and edit metadata in the local database (not in immich!)"
)


def prompt_with_default(text: str, default: str | None = None) -> str:
    """Prompt with a default value that can be edited using prompt_toolkit."""
    from prompt_toolkit import prompt
    from prompt_toolkit.key_binding import KeyBindings
    from prompt_toolkit.keys import Keys

    # For yes/no prompts with single character default
    if default in ("y", "n"):
        while True:
            try:
                click.echo(f"{text} (y/n) ", nl=False)
                char = click.getchar().lower()
                click.echo(char)  # Echo the character
                if char in ("y", "n"):
                    return char
                elif char in ("\r", "\n"):  # Enter
                    return default
            except (KeyboardInterrupt, EOFError):
                click.echo("\nOperation cancelled", err=True)
                raise

    # Create custom key bindings for editing
    kb = KeyBindings()

    @kb.add(Keys.Enter, eager=True)
    def _(event):
        """Accept the input."""
        event.current_buffer.validate_and_handle()

    try:
        # Show the prompt with the default value pre-filled and editable
        if default is not None:
            result = prompt(
                f"{text} ",
                default=default or "",
                key_bindings=kb,
            )
            return result if result.strip() else default
        else:
            return prompt(f"{text}: ")

    except (KeyboardInterrupt, EOFError) as e:
        click.echo("\nOperation cancelled", err=True)
        logger.debug(traceback(e))
        raise


console = Console()


@cli_db.command()
@logging_options
def init(log_level: str):
    """Initialize the database."""
    init_database()


@cli_db.command()
@logging_options
@click.option(
    "-i",
    "--not-finished",
    is_flag=True,
    help="Show only albums that are not fully processed",
)
@click.option(
    "-a",
    "--not-saved",
    is_flag=True,
    help="Show only albums that are not fully saved",
)
def show_albums(not_finished, not_saved, log_level: str):
    """Show albums in the database."""
    format = "table"  # TODO: add export
    with get_db_session() as session:
        albums = get_albums_from_db(
            session, not_finished=not_finished, not_saved=not_saved
        )

        if not albums:
            msg = "No albums found"
            if not_finished:
                msg += " that are not fully processed"
            console.print(f"[yellow]{msg} in database[/yellow]")
            return

        table_title = "Albums"
        if not_finished:
            table_title += " (Not Fully Processed)"

        table = Table(title=table_title)
        table.add_column("ID", style="cyan")
        table.add_column("Title", style="magenta")
        table.add_column("Items", style="green")
        table.add_column("Processed", style="yellow")
        table.add_column("Shared", style="red")
        table.add_column("Saved", style="red")
        table.add_column("Created", style="dim")

        true_sign = "✓" if format == "table" else True
        false_sign = "✗" if format == "table" else False
        for album in albums:
            # Calculate percentage with floor to avoid showing 100% until fully processed
            percentage = (
                math.floor((album.processed_items / album.items) * 100)
                if album.items > 0
                else 0
            )
            percentage_str = f"({percentage}%)"

            # Create clickable title with URL if available
            title_text = Text(album.title)
            if hasattr(album, "url") and album.url:
                title_text.stylize(f"link {album.url}")
                # title_text.append(" 🔗", style=Style(dim=True))

            table.add_row(
                str(album.album_id or "N/A"),
                title_text,
                str(album.items),
                f"{album.processed_items} [dim]{percentage_str}[/]",
                true_sign if album.shared else false_sign,
                true_sign if album.all_photos_saved else false_sign,
                str(album.created_at)[:19] if album.created_at else "N/A",
            )

        console.print(table)


@cli_db.command()
@click.option(
    "-i",
    "--immich",
    is_flag=True,
    help="Show only users which are set to be added to immich",
)
@click.option("-p", "--show-password", is_flag=True, help="Show inital password")
@click.option(
    "-f",
    "--format",
    type=click.Choice(["table", "csv", "json"]),
    default="table",
    help="Output format",
)
@logging_options
def show_users(immich: bool, show_password: bool, format: str, log_level: str):
    """Show all users in the database."""
    with get_db_session() as session:
        users = get_users_from_db(session)

        if not users:
            console.print("[yellow]No users found in the database.[/yellow]")
            return

        if format == "table":
            table = Table(title="Users in Database")
            table.add_column("ID", style="cyan")
            table.add_column("Source Name", style="magenta")
            table.add_column("Immich Name", style="green")
            table.add_column("Email", style="yellow")
            table.add_column("Immich ID", style="cyan")
            table.add_column("Immich", style="green")
            if show_password:
                table.add_column("Initial Password", style="dim green")
            table.add_column("Created At", style="dim")
        else:
            table: list[list[str]] = []
            header = [
                "id",
                "source_name",
                "immich_name",
                "email",
                "immich_id",
                "immich",
            ]
            if show_password:
                header.append("initial_password")
            header.append("created_at")

        true_sign = "✓" if format == "table" else True
        false_sign = "✗" if format == "table" else False
        none_sign = "✗" if format == "table" else None

        for user in users:
            if immich:
                if not user.add_to_immich:
                    continue
            row = [
                str(user.id),
                f"[strike]{user.source_name}[/]"
                if not user.add_to_immich
                else user.source_name,
                user.immich_name or none_sign,
                user.immich_email or none_sign,
                str(user.immich_user_id)
                if user.immich_user_id is not None
                else none_sign,
                true_sign if user.add_to_immich else false_sign,
            ]
            if show_password:
                row.append(user.immich_initial_password or none_sign)
            row.append(str(user.created_at)[:19] if user.created_at else "N/A")
            if format == "table":
                assert isinstance(table, Table)
                table.add_row(*row)
            else:
                assert isinstance(table, list)
                table.append(row)  # type: ignore

        if format == "table":
            assert isinstance(table, Table)
            console.print(table)
        elif format == "csv":
            assert isinstance(table, list)
            console.print(",".join(header))
            for row in table:
                click.echo(",".join(map(format_csv_value, row)))  # type: ignore
        elif format == "json":
            assert isinstance(table, list)
            table_json = [dict(zip(header, row)) for row in table]  # type: ignore
            click.echo(json.dumps(table_json, indent=2))


def update_user_immich_name(
    session: Session, user_id: int, immich_name: str | None = None
) -> None:
    """Update a user's immich name."""
    user = session.query(User).filter_by(id=user_id).first()
    if user:
        user.immich_name = immich_name or None
        session.commit()


def update_user_email(session: Session, user_id: int, email: str) -> None:
    """Update a user's email."""
    user = session.query(User).filter_by(id=user_id).first()
    if user:
        user.immich_email = email or None
        session.commit()


def update_user_add_to_immich(
    session: Session, user_id: int, add_to_immich: bool
) -> None:
    """Update whether to include user in Immich imports."""
    user = session.query(User).filter_by(id=user_id).first()
    if user:
        user.add_to_immich = add_to_immich
        session.commit()


@cli_db.command()
@click.option(
    "-d",
    "--domain",
    type=str,
    default=None,
    help="Domain to use for email generation (e.g., example.com)",
)
@click.option(
    "-a",
    "--all",
    is_flag=True,
    default=False,
    help="Show all users, including those with email already set",
)
@click.option(
    "-u",
    "--user-id",
    type=int,
    help="Edit a specific user by ID",
)
@logging_options
def edit_users(
    domain: str | None = None,
    all: bool = False,
    user_id: int | None = None,
    log_level: str = "INFO",
):
    """Interactively edit user information in the database.

    By default, only shows users added to Immich without an email.
    Use --all to show all users, or --user-id to edit a specific user.
    """
    with get_db_session() as session:
        if user_id is not None:
            # Edit specific user by ID
            user = session.query(User).filter_by(id=user_id).first()
            if not user:
                console.print(f"[red]User with ID {user_id} not found[/red]")
                return
            users = [user]
            all = True  # Show the user even if they have an email or are not added to Immich
        else:
            # Get all users
            users = get_users_from_db(session)
            if not users:
                console.print("[yellow]No users found in database[/yellow]")
                return

            # Filter users if --all is not set
            if not all:
                # Show only users added to Immich and without email by default
                filtered_users = [
                    u for u in users if u.add_to_immich and not u.immich_email
                ]
                if filtered_users:
                    users = filtered_users
                    console.print(
                        "[yellow]Showing only users added to Immich without email (use --all to show all users)[/yellow]"
                    )
                else:
                    users = []
                    console.print(
                        "[red]All users processed (use [yellow]--all[/yellow] to show all users)[/red]"
                    )

        # Always run in interactive mode
        for user in users:
            console.print("\n" + "─" * 50)
            console.print(
                f"User [cyan]{user.id}[/] - Source: [magenta]{user.source_name}[/]"
            )

            try:
                # Toggle add_to_immich
                current_status = "yes" if user.add_to_immich else "no"
                enable = prompt_with_default(
                    f"Include in Immich? [current: {current_status}]",
                    "y" if user.add_to_immich else "n",
                )

                if enable == "y":
                    # Edit name - use source_name as default if immich_name is not set
                    current_name = (
                        user.immich_name
                        if user.immich_name is not None
                        else user.source_name
                    )
                    new_name = prompt_with_default("  Immich name: ", current_name)
                    # If user enters a single dot, clear the name
                    if new_name.strip() == ".":
                        new_name = ""
                    if new_name != user.immich_name:
                        update_user_immich_name(
                            session, user.id, new_name if new_name.strip() else None
                        )
                        console.print(
                            f"  → Updated name to: [green]{new_name if new_name else '✗'}[/]"
                        )

                if enable.lower() == "y" or (enable == "" and user.add_to_immich):
                    # User is enabled for Immich
                    update_user_add_to_immich(session, user.id, True)

                    # Edit email
                    email_default = user.immich_email or ""

                    # Generate email proposal if domain is provided
                    if domain and not email_default:
                        # Use the current immich_name (which might have just been updated) or source_name
                        name_to_use = user.immich_name or user.source_name
                        email_local = sanitize_for_email(name_to_use)
                        email_default = f"{email_local}@{domain}"

                    new_email = prompt_with_default("  Email: ", email_default)
                    if new_email != user.immich_email:
                        update_user_email(session, user.id, new_email)
                        console.print(f"  → Updated email to: [yellow]{new_email}[/]")

                elif enable.lower() == "n" or (enable == "" and not user.add_to_immich):
                    # User is disabled for Immich
                    update_user_add_to_immich(session, user.id, False)
                    if user.immich_name:
                        update_user_immich_name(session, user.id, None)
                    console.print("  [yellow]User disabled for Immich import[/]")

                # Add a small space between users
                console.print()

            except (KeyboardInterrupt, EOFError):
                if click.confirm("\nDo you want to stop editing?"):
                    break
                continue

        console.print(
            "Run [yellow]'immichporter db show-users'[/yellow] to see the updated users."
        )
        # show_users.callback()


@cli_db.command()
@logging_options
def show_stats(log_level: str):
    """Show database statistics."""
    with get_db_session() as session:
        stats = get_database_stats(session)

        # Create a table for statistics
        table = Table(show_header=True, header_style="bold magenta")
        table.add_column("Statistic", style="dim", width=30)
        table.add_column("Count", justify="right")

        # Add database stats
        table.add_row("Total Albums", str(stats["total_albums"]))
        table.add_row("Total Photos", str(stats["total_photos"]))
        table.add_row("Total Users", str(stats["total_users"]))
        table.add_row("Total Errors", str(stats["total_errors"]))

        # Add album stats if available
        if "album_stats" in stats:
            table.add_section()
            table.add_row("[bold]Album Statistics[/bold]", "")
            for album_name, count in stats["album_stats"].items():
                table.add_row(f"  {album_name}", str(count))

        # Add user stats if available
        if "user_stats" in stats:
            table.add_section()
            table.add_row("[bold]User Statistics[/bold]", "")
            for user_name, count in stats["user_stats"].items():
                table.add_row(f"  {user_name}", str(count))

        console.print(table)


def drop_table(name: str, all_tables: bool = False, force: bool = False):
    """Drop database tables or the entire database file.

    Args:
        name: Name of the table to drop (albums, photos, users, errors)
        all_tables: If True, drop all tables
        force: If True, don't ask for confirmation
    """
    if not any([name, all_tables]):
        console.print("[red]Error: Please specify a table name or use --all")
        return

    # Map table names to their models
    table_models = {"albums": Album, "photos": Photo, "users": User, "errors": Error}

    if all_tables:
        if not force:
            if not click.confirm(
                "Are you sure you want to drop ALL tables? This cannot be undone."
            ):
                console.print("Operation cancelled.")
                return

        with get_db_session() as session:
            try:
                # Drop all tables
                engine = SessionLocal().bind
                Base.metadata.drop_all(bind=engine)
                console.print("[green]All tables have been dropped.")

                # Recreate all tables
                Base.metadata.create_all(bind=engine)
                console.print("A new empty database has been initialized.")

            except Exception as e:
                console.print(f"[red]Error dropping tables: {str(e)}")
                logger.debug(traceback(e))
        return

    if name not in table_models:
        console.print(
            f"[red]Error: Invalid table name: {name}. Must be one of: {', '.join(table_models.keys())}"
        )
        return

    if not force:
        if not click.confirm(
            f"Are you sure you want to drop the '{name}' table? This cannot be undone."
        ):
            console.print("Operation cancelled.")
            return

    with get_db_session() as session:
        try:
            table = table_models[name].__table__

            # SQLite doesn't support CASCADE in DROP TABLE
            # First disable foreign key constraints
            session.execute(text("PRAGMA foreign_keys = OFF"))

            # If dropping albums, first delete all photos and errors that reference albums
            if name == "albums":
                session.execute(text("DELETE FROM photos"))
                session.execute(text("DELETE FROM errors"))

            # Drop the table
            session.execute(text(f"DROP TABLE IF EXISTS {table.name}"))

            # Re-enable foreign key constraints
            session.execute(text("PRAGMA foreign_keys = ON"))
            session.commit()

            console.print(f"[green]Table '{name}' has been dropped.")

            # Recreate the table
            engine = SessionLocal().bind
            Base.metadata.create_all(bind=engine, tables=[table])
            console.print(f"Table '{name}' has been recreated (empty).")

        except SQLAlchemyError as e:
            session.rollback()
            console.print(f"[red]Error dropping table: {str(e)}")
            logger.debug(traceback(e))
        except Exception as e:
            session.rollback()
            console.print(f"[red]Unexpected error: {str(e)}")
            logger.debug(traceback(e))


@cli_db.command("drop")
@click.option(
    "-n", "--name", help="Name of the table to drop (albums, photos, users, errors)"
)
@click.option(
    "-a",
    "--all",
    "all_tables",
    is_flag=True,
    help="Drop all tables and recreate the database",
)
@click.option("-f", "--force", "force", is_flag=True, help="Skip confirmation prompt")
@logging_options
@click.pass_context
def drop_command(ctx, name: str, all_tables: bool, force: bool, log_level: str):
    """Drop database tables or the entire database."""
    drop_table(name, all_tables, force)



import datetime
from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
    cast,
)
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.notification_level import NotificationLevel
from ..models.notification_type import NotificationType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.notification_create_dto_data import NotificationCreateDtoData


T = TypeVar("T", bound="NotificationCreateDto")


@_attrs_define
class NotificationCreateDto:
    """
    Attributes:
        title (str):
        user_id (UUID):
        data (Union[Unset, NotificationCreateDtoData]):
        description (Union[None, Unset, str]):
        level (Union[Unset, NotificationLevel]):
        read_at (Union[None, Unset, datetime.datetime]):
        type_ (Union[Unset, NotificationType]):
    """

    title: str
    user_id: UUID
    data: Union[Unset, "NotificationCreateDtoData"] = UNSET
    description: Union[None, Unset, str] = UNSET
    level: Union[Unset, NotificationLevel] = UNSET
    read_at: Union[None, Unset, datetime.datetime] = UNSET
    type_: Union[Unset, NotificationType] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        title = self.title

        user_id = str(self.user_id)

        data: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.data, Unset):
            data = self.data.to_dict()

        description: Union[None, Unset, str]
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        level: Union[Unset, str] = UNSET
        if not isinstance(self.level, Unset):
            level = self.level.value

        read_at: Union[None, Unset, str]
        if isinstance(self.read_at, Unset):
            read_at = UNSET
        elif isinstance(self.read_at, datetime.datetime):
            read_at = self.read_at.isoformat()
        else:
            read_at = self.read_at

        type_: Union[Unset, str] = UNSET
        if not isinstance(self.type_, Unset):
            type_ = self.type_.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "title": title,
                "userId": user_id,
            }
        )
        if data is not UNSET:
            field_dict["data"] = data
        if description is not UNSET:
            field_dict["description"] = description
        if level is not UNSET:
            field_dict["level"] = level
        if read_at is not UNSET:
            field_dict["readAt"] = read_at
        if type_ is not UNSET:
            field_dict["type"] = type_

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.notification_create_dto_data import NotificationCreateDtoData

        d = dict(src_dict)
        title = d.pop("title")

        user_id = UUID(d.pop("userId"))

        _data = d.pop("data", UNSET)
        data: Union[Unset, NotificationCreateDtoData]
        if isinstance(_data, Unset):
            data = UNSET
        else:
            data = NotificationCreateDtoData.from_dict(_data)

        def _parse_description(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        description = _parse_description(d.pop("description", UNSET))

        _level = d.pop("level", UNSET)
        level: Union[Unset, NotificationLevel]
        if isinstance(_level, Unset):
            level = UNSET
        else:
            level = NotificationLevel(_level)

        def _parse_read_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                read_at_type_0 = isoparse(data)

                return read_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        read_at = _parse_read_at(d.pop("readAt", UNSET))

        _type_ = d.pop("type", UNSET)
        type_: Union[Unset, NotificationType]
        if isinstance(_type_, Unset):
            type_ = UNSET
        else:
            type_ = NotificationType(_type_)

        notification_create_dto = cls(
            title=title,
            user_id=user_id,
            data=data,
            description=description,
            level=level,
            read_at=read_at,
            type_=type_,
        )

        notification_create_dto.additional_properties = d
        return notification_create_dto

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="PurchaseResponse")


@_attrs_define
class PurchaseResponse:
    """
    Attributes:
        hide_buy_button_until (str):
        show_support_badge (bool):
    """

    hide_buy_button_until: str
    show_support_badge: bool
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        hide_buy_button_until = self.hide_buy_button_until

        show_support_badge = self.show_support_badge

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "hideBuyButtonUntil": hide_buy_button_until,
                "showSupportBadge": show_support_badge,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        hide_buy_button_until = d.pop("hideBuyButtonUntil")

        show_support_badge = d.pop("showSupportBadge")

        purchase_response = cls(
            hide_buy_button_until=hide_buy_button_until,
            show_support_badge=show_support_badge,
        )

        purchase_response.additional_properties = d
        return purchase_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.reaction_type import ReactionType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.user_response_dto import UserResponseDto


T = TypeVar("T", bound="ActivityResponseDto")


@_attrs_define
class ActivityResponseDto:
    """
    Attributes:
        asset_id (Union[None, str]):
        created_at (datetime.datetime):
        id (str):
        type_ (ReactionType):
        user (UserResponseDto):
        comment (Union[None, Unset, str]):
    """

    asset_id: Union[None, str]
    created_at: datetime.datetime
    id: str
    type_: ReactionType
    user: "UserResponseDto"
    comment: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        asset_id: Union[None, str]
        asset_id = self.asset_id

        created_at = self.created_at.isoformat()

        id = self.id

        type_ = self.type_.value

        user = self.user.to_dict()

        comment: Union[None, Unset, str]
        if isinstance(self.comment, Unset):
            comment = UNSET
        else:
            comment = self.comment

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "assetId": asset_id,
                "createdAt": created_at,
                "id": id,
                "type": type_,
                "user": user,
            }
        )
        if comment is not UNSET:
            field_dict["comment"] = comment

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.user_response_dto import UserResponseDto

        d = dict(src_dict)

        def _parse_asset_id(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        asset_id = _parse_asset_id(d.pop("assetId"))

        created_at = isoparse(d.pop("createdAt"))

        id = d.pop("id")

        type_ = ReactionType(d.pop("type"))

        user = UserResponseDto.from_dict(d.pop("user"))

        def _parse_comment(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        comment = _parse_comment(d.pop("comment", UNSET))

        activity_response_dto = cls(
            asset_id=asset_id,
            created_at=created_at,
            id=id,
            type_=type_,
            user=user,
            comment=comment,
        )

        activity_response_dto.additional_properties = d
        return activity_response_dto

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.memory_type import MemoryType

if TYPE_CHECKING:
    from ..models.sync_memory_v1_data import SyncMemoryV1Data


T = TypeVar("T", bound="SyncMemoryV1")


@_attrs_define
class SyncMemoryV1:
    """
    Attributes:
        created_at (datetime.datetime):
        data (SyncMemoryV1Data):
        deleted_at (Union[None, datetime.datetime]):
        hide_at (Union[None, datetime.datetime]):
        id (str):
        is_saved (bool):
        memory_at (datetime.datetime):
        owner_id (str):
        seen_at (Union[None, datetime.datetime]):
        show_at (Union[None, datetime.datetime]):
        type_ (MemoryType):
        updated_at (datetime.datetime):
    """

    created_at: datetime.datetime
    data: "SyncMemoryV1Data"
    deleted_at: Union[None, datetime.datetime]
    hide_at: Union[None, datetime.datetime]
    id: str
    is_saved: bool
    memory_at: datetime.datetime
    owner_id: str
    seen_at: Union[None, datetime.datetime]
    show_at: Union[None, datetime.datetime]
    type_: MemoryType
    updated_at: datetime.datetime
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        created_at = self.created_at.isoformat()

        data = self.data.to_dict()

        deleted_at: Union[None, str]
        if isinstance(self.deleted_at, datetime.datetime):
            deleted_at = self.deleted_at.isoformat()
        else:
            deleted_at = self.deleted_at

        hide_at: Union[None, str]
        if isinstance(self.hide_at, datetime.datetime):
            hide_at = self.hide_at.isoformat()
        else:
            hide_at = self.hide_at

        id = self.id

        is_saved = self.is_saved

        memory_at = self.memory_at.isoformat()

        owner_id = self.owner_id

        seen_at: Union[None, str]
        if isinstance(self.seen_at, datetime.datetime):
            seen_at = self.seen_at.isoformat()
        else:
            seen_at = self.seen_at

        show_at: Union[None, str]
        if isinstance(self.show_at, datetime.datetime):
            show_at = self.show_at.isoformat()
        else:
            show_at = self.show_at

        type_ = self.type_.value

        updated_at = self.updated_at.isoformat()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "createdAt": created_at,
                "data": data,
                "deletedAt": deleted_at,
                "hideAt": hide_at,
                "id": id,
                "isSaved": is_saved,
                "memoryAt": memory_at,
                "ownerId": owner_id,
                "seenAt": seen_at,
                "showAt": show_at,
                "type": type_,
                "updatedAt": updated_at,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.sync_memory_v1_data import SyncMemoryV1Data

        d = dict(src_dict)
        created_at = isoparse(d.pop("createdAt"))

        data = SyncMemoryV1Data.from_dict(d.pop("data"))

        def _parse_deleted_at(data: object) -> Union[None, datetime.datetime]:
            if data is None:
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                deleted_at_type_0 = isoparse(data)

                return deleted_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, datetime.datetime], data)

        deleted_at = _parse_deleted_at(d.pop("deletedAt"))

        def _parse_hide_at(data: object) -> Union[None, datetime.datetime]:
            if data is None:
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                hide_at_type_0 = isoparse(data)

                return hide_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, datetime.datetime], data)

        hide_at = _parse_hide_at(d.pop("hideAt"))

        id = d.pop("id")

        is_saved = d.pop("isSaved")

        memory_at = isoparse(d.pop("memoryAt"))

        owner_id = d.pop("ownerId")

        def _parse_seen_at(data: object) -> Union[None, datetime.datetime]:
            if data is None:
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                seen_at_type_0 = isoparse(data)

                return seen_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, datetime.datetime], data)

        seen_at = _parse_seen_at(d.pop("seenAt"))

        def _parse_show_at(data: object) -> Union[None, datetime.datetime]:
            if data is None:
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                show_at_type_0 = isoparse(data)

                return show_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, datetime.datetime], data)

        show_at = _parse_show_at(d.pop("showAt"))

        type_ = MemoryType(d.pop("type"))

        updated_at = isoparse(d.pop("updatedAt"))

        sync_memory_v1 = cls(
            created_at=created_at,
            data=data,
            deleted_at=deleted_at,
            hide_at=hide_at,
            id=id,
            is_saved=is_saved,
            memory_at=memory_at,
            owner_id=owner_id,
            seen_at=seen_at,
            show_at=show_at,
            type_=type_,
            updated_at=updated_at,
        )

        sync_memory_v1.additional_properties = d
        return sync_memory_v1

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.map_marker_response_dto import MapMarkerResponseDto
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    is_archived: Union[Unset, bool] = UNSET,
    is_favorite: Union[Unset, bool] = UNSET,
    file_created_after: Union[Unset, datetime.datetime] = UNSET,
    file_created_before: Union[Unset, datetime.datetime] = UNSET,
    with_partners: Union[Unset, bool] = UNSET,
    with_shared_albums: Union[Unset, bool] = UNSET,
) -> dict[str, Any]:
    params: dict[str, Any] = {}

    params["isArchived"] = is_archived

    params["isFavorite"] = is_favorite

    json_file_created_after: Union[Unset, str] = UNSET
    if not isinstance(file_created_after, Unset):
        json_file_created_after = file_created_after.isoformat()
    params["fileCreatedAfter"] = json_file_created_after

    json_file_created_before: Union[Unset, str] = UNSET
    if not isinstance(file_created_before, Unset):
        json_file_created_before = file_created_before.isoformat()
    params["fileCreatedBefore"] = json_file_created_before

    params["withPartners"] = with_partners

    params["withSharedAlbums"] = with_shared_albums

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/map/markers",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[list["MapMarkerResponseDto"]]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = MapMarkerResponseDto.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[list["MapMarkerResponseDto"]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    is_archived: Union[Unset, bool] = UNSET,
    is_favorite: Union[Unset, bool] = UNSET,
    file_created_after: Union[Unset, datetime.datetime] = UNSET,
    file_created_before: Union[Unset, datetime.datetime] = UNSET,
    with_partners: Union[Unset, bool] = UNSET,
    with_shared_albums: Union[Unset, bool] = UNSET,
) -> Response[list["MapMarkerResponseDto"]]:
    """
    Args:
        is_archived (Union[Unset, bool]):
        is_favorite (Union[Unset, bool]):
        file_created_after (Union[Unset, datetime.datetime]):
        file_created_before (Union[Unset, datetime.datetime]):
        with_partners (Union[Unset, bool]):
        with_shared_albums (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[list['MapMarkerResponseDto']]
    """

    kwargs = _get_kwargs(
        is_archived=is_archived,
        is_favorite=is_favorite,
        file_created_after=file_created_after,
        file_created_before=file_created_before,
        with_partners=with_partners,
        with_shared_albums=with_shared_albums,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    is_archived: Union[Unset, bool] = UNSET,
    is_favorite: Union[Unset, bool] = UNSET,
    file_created_after: Union[Unset, datetime.datetime] = UNSET,
    file_created_before: Union[Unset, datetime.datetime] = UNSET,
    with_partners: Union[Unset, bool] = UNSET,
    with_shared_albums: Union[Unset, bool] = UNSET,
) -> Optional[list["MapMarkerResponseDto"]]:
    """
    Args:
        is_archived (Union[Unset, bool]):
        is_favorite (Union[Unset, bool]):
        file_created_after (Union[Unset, datetime.datetime]):
        file_created_before (Union[Unset, datetime.datetime]):
        with_partners (Union[Unset, bool]):
        with_shared_albums (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        list['MapMarkerResponseDto']
    """

    return sync_detailed(
        client=client,
        is_archived=is_archived,
        is_favorite=is_favorite,
        file_created_after=file_created_after,
        file_created_before=file_created_before,
        with_partners=with_partners,
        with_shared_albums=with_shared_albums,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    is_archived: Union[Unset, bool] = UNSET,
    is_favorite: Union[Unset, bool] = UNSET,
    file_created_after: Union[Unset, datetime.datetime] = UNSET,
    file_created_before: Union[Unset, datetime.datetime] = UNSET,
    with_partners: Union[Unset, bool] = UNSET,
    with_shared_albums: Union[Unset, bool] = UNSET,
) -> Response[list["MapMarkerResponseDto"]]:
    """
    Args:
        is_archived (Union[Unset, bool]):
        is_favorite (Union[Unset, bool]):
        file_created_after (Union[Unset, datetime.datetime]):
        file_created_before (Union[Unset, datetime.datetime]):
        with_partners (Union[Unset, bool]):
        with_shared_albums (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[list['MapMarkerResponseDto']]
    """

    kwargs = _get_kwargs(
        is_archived=is_archived,
        is_favorite=is_favorite,
        file_created_after=file_created_after,
        file_created_before=file_created_before,
        with_partners=with_partners,
        with_shared_albums=with_shared_albums,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    is_archived: Union[Unset, bool] = UNSET,
    is_favorite: Union[Unset, bool] = UNSET,
    file_created_after: Union[Unset, datetime.datetime] = UNSET,
    file_created_before: Union[Unset, datetime.datetime] = UNSET,
    with_partners: Union[Unset, bool] = UNSET,
    with_shared_albums: Union[Unset, bool] = UNSET,
) -> Optional[list["MapMarkerResponseDto"]]:
    """
    Args:
        is_archived (Union[Unset, bool]):
        is_favorite (Union[Unset, bool]):
        file_created_after (Union[Unset, datetime.datetime]):
        file_created_before (Union[Unset, datetime.datetime]):
        with_partners (Union[Unset, bool]):
        with_shared_albums (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        list['MapMarkerResponseDto']
    """

    return (
        await asyncio_detailed(
            client=client,
            is_archived=is_archived,
            is_favorite=is_favorite,
            file_created_after=file_created_after,
            file_created_before=file_created_before,
            with_partners=with_partners,
            with_shared_albums=with_shared_albums,
        )
    ).parsed



from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="ServerApkLinksDto")


@_attrs_define
class ServerApkLinksDto:
    """
    Attributes:
        arm64v8a (str):
        armeabiv7a (str):
        universal (str):
        x86_64 (str):
    """

    arm64v8a: str
    armeabiv7a: str
    universal: str
    x86_64: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        arm64v8a = self.arm64v8a

        armeabiv7a = self.armeabiv7a

        universal = self.universal

        x86_64 = self.x86_64

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "arm64v8a": arm64v8a,
                "armeabiv7a": armeabiv7a,
                "universal": universal,
                "x86_64": x86_64,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        arm64v8a = d.pop("arm64v8a")

        armeabiv7a = d.pop("armeabiv7a")

        universal = d.pop("universal")

        x86_64 = d.pop("x86_64")

        server_apk_links_dto = cls(
            arm64v8a=arm64v8a,
            armeabiv7a=armeabiv7a,
            universal=universal,
            x86_64=x86_64,
        )

        server_apk_links_dto.additional_properties = d
        return server_apk_links_dto

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class UserAvatarColor(str, Enum):
    AMBER = "amber"
    BLUE = "blue"
    GRAY = "gray"
    GREEN = "green"
    ORANGE = "orange"
    PINK = "pink"
    PRIMARY = "primary"
    PURPLE = "purple"
    RED = "red"
    YELLOW = "yellow"

    def __str__(self) -> str:
        return str(self.value)



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.system_config_backups_dto import SystemConfigBackupsDto
    from ..models.system_config_f_fmpeg_dto import SystemConfigFFmpegDto
    from ..models.system_config_image_dto import SystemConfigImageDto
    from ..models.system_config_job_dto import SystemConfigJobDto
    from ..models.system_config_library_dto import SystemConfigLibraryDto
    from ..models.system_config_logging_dto import SystemConfigLoggingDto
    from ..models.system_config_machine_learning_dto import (
        SystemConfigMachineLearningDto,
    )
    from ..models.system_config_map_dto import SystemConfigMapDto
    from ..models.system_config_metadata_dto import SystemConfigMetadataDto
    from ..models.system_config_new_version_check_dto import (
        SystemConfigNewVersionCheckDto,
    )
    from ..models.system_config_nightly_tasks_dto import SystemConfigNightlyTasksDto
    from ..models.system_config_notifications_dto import SystemConfigNotificationsDto
    from ..models.system_config_o_auth_dto import SystemConfigOAuthDto
    from ..models.system_config_password_login_dto import SystemConfigPasswordLoginDto
    from ..models.system_config_reverse_geocoding_dto import (
        SystemConfigReverseGeocodingDto,
    )
    from ..models.system_config_server_dto import SystemConfigServerDto
    from ..models.system_config_storage_template_dto import (
        SystemConfigStorageTemplateDto,
    )
    from ..models.system_config_templates_dto import SystemConfigTemplatesDto
    from ..models.system_config_theme_dto import SystemConfigThemeDto
    from ..models.system_config_trash_dto import SystemConfigTrashDto
    from ..models.system_config_user_dto import SystemConfigUserDto


T = TypeVar("T", bound="SystemConfigDto")


@_attrs_define
class SystemConfigDto:
    """
    Attributes:
        backup (SystemConfigBackupsDto):
        ffmpeg (SystemConfigFFmpegDto):
        image (SystemConfigImageDto):
        job (SystemConfigJobDto):
        library (SystemConfigLibraryDto):
        logging (SystemConfigLoggingDto):
        machine_learning (SystemConfigMachineLearningDto):
        map_ (SystemConfigMapDto):
        metadata (SystemConfigMetadataDto):
        new_version_check (SystemConfigNewVersionCheckDto):
        nightly_tasks (SystemConfigNightlyTasksDto):
        notifications (SystemConfigNotificationsDto):
        oauth (SystemConfigOAuthDto):
        password_login (SystemConfigPasswordLoginDto):
        reverse_geocoding (SystemConfigReverseGeocodingDto):
        server (SystemConfigServerDto):
        storage_template (SystemConfigStorageTemplateDto):
        templates (SystemConfigTemplatesDto):
        theme (SystemConfigThemeDto):
        trash (SystemConfigTrashDto):
        user (SystemConfigUserDto):
    """

    backup: "SystemConfigBackupsDto"
    ffmpeg: "SystemConfigFFmpegDto"
    image: "SystemConfigImageDto"
    job: "SystemConfigJobDto"
    library: "SystemConfigLibraryDto"
    logging: "SystemConfigLoggingDto"
    machine_learning: "SystemConfigMachineLearningDto"
    map_: "SystemConfigMapDto"
    metadata: "SystemConfigMetadataDto"
    new_version_check: "SystemConfigNewVersionCheckDto"
    nightly_tasks: "SystemConfigNightlyTasksDto"
    notifications: "SystemConfigNotificationsDto"
    oauth: "SystemConfigOAuthDto"
    password_login: "SystemConfigPasswordLoginDto"
    reverse_geocoding: "SystemConfigReverseGeocodingDto"
    server: "SystemConfigServerDto"
    storage_template: "SystemConfigStorageTemplateDto"
    templates: "SystemConfigTemplatesDto"
    theme: "SystemConfigThemeDto"
    trash: "SystemConfigTrashDto"
    user: "SystemConfigUserDto"
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        backup = self.backup.to_dict()

        ffmpeg = self.ffmpeg.to_dict()

        image = self.image.to_dict()

        job = self.job.to_dict()

        library = self.library.to_dict()

        logging = self.logging.to_dict()

        machine_learning = self.machine_learning.to_dict()

        map_ = self.map_.to_dict()

        metadata = self.metadata.to_dict()

        new_version_check = self.new_version_check.to_dict()

        nightly_tasks = self.nightly_tasks.to_dict()

        notifications = self.notifications.to_dict()

        oauth = self.oauth.to_dict()

        password_login = self.password_login.to_dict()

        reverse_geocoding = self.reverse_geocoding.to_dict()

        server = self.server.to_dict()

        storage_template = self.storage_template.to_dict()

        templates = self.templates.to_dict()

        theme = self.theme.to_dict()

        trash = self.trash.to_dict()

        user = self.user.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "backup": backup,
                "ffmpeg": ffmpeg,
                "image": image,
                "job": job,
                "library": library,
                "logging": logging,
                "machineLearning": machine_learning,
                "map": map_,
                "metadata": metadata,
                "newVersionCheck": new_version_check,
                "nightlyTasks": nightly_tasks,
                "notifications": notifications,
                "oauth": oauth,
                "passwordLogin": password_login,
                "reverseGeocoding": reverse_geocoding,
                "server": server,
                "storageTemplate": storage_template,
                "templates": templates,
                "theme": theme,
                "trash": trash,
                "user": user,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.system_config_backups_dto import SystemConfigBackupsDto
        from ..models.system_config_f_fmpeg_dto import SystemConfigFFmpegDto
        from ..models.system_config_image_dto import SystemConfigImageDto
        from ..models.system_config_job_dto import SystemConfigJobDto
        from ..models.system_config_library_dto import SystemConfigLibraryDto
        from ..models.system_config_logging_dto import SystemConfigLoggingDto
        from ..models.system_config_machine_learning_dto import (
            SystemConfigMachineLearningDto,
        )
        from ..models.system_config_map_dto import SystemConfigMapDto
        from ..models.system_config_metadata_dto import SystemConfigMetadataDto
        from ..models.system_config_new_version_check_dto import (
            SystemConfigNewVersionCheckDto,
        )
        from ..models.system_config_nightly_tasks_dto import SystemConfigNightlyTasksDto
        from ..models.system_config_notifications_dto import (
            SystemConfigNotificationsDto,
        )
        from ..models.system_config_o_auth_dto import SystemConfigOAuthDto
        from ..models.system_config_password_login_dto import (
            SystemConfigPasswordLoginDto,
        )
        from ..models.system_config_reverse_geocoding_dto import (
            SystemConfigReverseGeocodingDto,
        )
        from ..models.system_config_server_dto import SystemConfigServerDto
        from ..models.system_config_storage_template_dto import (
            SystemConfigStorageTemplateDto,
        )
        from ..models.system_config_templates_dto import SystemConfigTemplatesDto
        from ..models.system_config_theme_dto import SystemConfigThemeDto
        from ..models.system_config_trash_dto import SystemConfigTrashDto
        from ..models.system_config_user_dto import SystemConfigUserDto

        d = dict(src_dict)
        backup = SystemConfigBackupsDto.from_dict(d.pop("backup"))

        ffmpeg = SystemConfigFFmpegDto.from_dict(d.pop("ffmpeg"))

        image = SystemConfigImageDto.from_dict(d.pop("image"))

        job = SystemConfigJobDto.from_dict(d.pop("job"))

        library = SystemConfigLibraryDto.from_dict(d.pop("library"))

        logging = SystemConfigLoggingDto.from_dict(d.pop("logging"))

        machine_learning = SystemConfigMachineLearningDto.from_dict(
            d.pop("machineLearning")
        )

        map_ = SystemConfigMapDto.from_dict(d.pop("map"))

        metadata = SystemConfigMetadataDto.from_dict(d.pop("metadata"))

        new_version_check = SystemConfigNewVersionCheckDto.from_dict(
            d.pop("newVersionCheck")
        )

        nightly_tasks = SystemConfigNightlyTasksDto.from_dict(d.pop("nightlyTasks"))

        notifications = SystemConfigNotificationsDto.from_dict(d.pop("notifications"))

        oauth = SystemConfigOAuthDto.from_dict(d.pop("oauth"))

        password_login = SystemConfigPasswordLoginDto.from_dict(d.pop("passwordLogin"))

        reverse_geocoding = SystemConfigReverseGeocodingDto.from_dict(
            d.pop("reverseGeocoding")
        )

        server = SystemConfigServerDto.from_dict(d.pop("server"))

        storage_template = SystemConfigStorageTemplateDto.from_dict(
            d.pop("storageTemplate")
        )

        templates = SystemConfigTemplatesDto.from_dict(d.pop("templates"))

        theme = SystemConfigThemeDto.from_dict(d.pop("theme"))

        trash = SystemConfigTrashDto.from_dict(d.pop("trash"))

        user = SystemConfigUserDto.from_dict(d.pop("user"))

        system_config_dto = cls(
            backup=backup,
            ffmpeg=ffmpeg,
            image=image,
            job=job,
            library=library,
            logging=logging,
            machine_learning=machine_learning,
            map_=map_,
            metadata=metadata,
            new_version_check=new_version_check,
            nightly_tasks=nightly_tasks,
            notifications=notifications,
            oauth=oauth,
            password_login=password_login,
            reverse_geocoding=reverse_geocoding,
            server=server,
            storage_template=storage_template,
            templates=templates,
            theme=theme,
            trash=trash,
            user=user,
        )

        system_config_dto.additional_properties = d
        return system_config_dto

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
